; OSIDISKTOOL
;
; OSI DISK TOOL IS A DISK SERIAL READER/WRITER UTILITY FOR OSI C1P/UK101/C2PDF/C4PMF/C8P/C3-OEM
; MARK SPANKUS 10/2020  http://osi.marks-lab.com
;
; BASED ON ED'S OSIDUMP UTILITY http://dxforth.mirrors.minimaltype.com/
; PIECES FROM DAVID GESSWEIN'S OSI DISK TEST  http://www.pdp8online.com/osi/osi-floppy-test.shtml
; INCORPORATED X-MODEM FILE TRANSFER PROTOCOL CODE BY DARYL RICTOR http://6502.org/source/io/xmodem/xmodem.htm
;
;
; OSI DISK TOOL IS DESIGNED TO BE CONTROLLED THROUGH THE OSI SERIAL PORT SINCE THE GOAL IS TO
; UPLOAD OR DOWNLOAD DISK DATA TO/FROM A REMOTE COMPUTER.  RECOMMENDED SPEED FOR THE OSI SERIAL PORT
; IS AT LEAST 9600 BAUD.  IF THE OSI IS WIRED FOR 300 BAUD, AN OPTIONAL 16X MODE IS AVAILABLE AT
; THE MAIN MENU WHICH WILL ALLOW TRANSFER AT 300*16 OR 4800 BAUD.
;
; OPERATION
; YOU CAN USE THIS TOOL ON AN OSI DISK SYSTEM WITH BLANK FLOPPIES TO CREATE A BOOTABLE OPERATING SYSTEM!
; BEGIN BY ENTERING THE ROM MONITOR ON THE OSI AFTER RESET, PRESS 'M' AT THE H/D/M OR D/C/W/M PROMPT
; ON VIDEO BASED SYSTEMS PRESS 'L' TO BEGIN LOAD, SEND DISKTOOL.LOD AS RAW ASCII FROM YOUR TERMINAL PROGRAM
; FOR SERIAL BASED SYSTEMS SEND DISKTOOL.65A AS RAW ASCII FROM YOUR TERMINAL PROGRAM, WHEN LOADED THE
;    PROGRAM SHOULD START AUTOMATICALLY, IF NOT <RESET>, PRESS 'M' AND THEN 'G' TO START
; ENSURE THE DISK TYPE MATCHES YOUR SYSTEM (5.25"/8"), INSERT A WRITABLE DISK IN THE DRIVE THEN
; SELECT '3. RECEIVE DISK XMODEM' FROM THE MENU TO BEGIN TRANSFER.  SEND THE DESIRED DISK IMAGE VIA
; XMODEM FROM YOUR TERMINAL PROGRAM.  
; VIDEO SYSTEMS WILL DISPLAY PROGRESS INFORMATION ON THE SCREEN DURING TRANSFER
; ALL SYSTEM CAN VIEW THE INFORMATION AFTER TRANSFER BY PRESSING L 
; GOOD LUCK!
;
;
; THIS VERSION ADDS SUPPORT FOR WRITING BINARY OSI DISK IMAGES VIA XMODEM & SERIAL PORT AS 
; WELL AS DUMPING THE CONTENTS OF A DISK BY SENDING OUTPUT TO THE SERIAL PORT AS ASCII HEX OR 
; BINARY XMODEM TRANSFERS. 
;
; RAW DUMP DUMPS EVERY BYTE ENCOUNTERED ON A TRACK STARTING AT THE INDEX PULSE.
;
; NOTE: OS TRACKS MAY CONTAIN SPURIOUS CHARACTERS BEFORE AND AFTER SECTORS AND CAN BE IGNORED.
; OSIDISKTOOL WILL REMOVE NOISE BYTES FROM UPLOADED TRACKS
; NOTE: OS65U DISKS SWITCH ENCODING FROM 8E1 TO 8N1 ON TRACKS 2-77 AFTER TRACK IDENTIFIER
; OS65D AND RELATED FORMATS USE 8E1 ON THE WHOLE DISK



; OSI DISK FORMAT SUMMARY
; OSI uses a 6850 ACIA to read/write bits to disk using FM encoding, generally starting at end of disk index pulse
; by default OSI uses 8E1 serial word encoding for 11bits/byte to detect data errors
; OS65U switches to 8N1 10bits/byte with checksum to pack more data per track (14 pages data + overhead)

;OSI Boot Track format (loader built into system ROM)
; trk0 [8E1]<index><delay>loadHi,loadLo,#pages,(#pages*256 data)  ;loadhi,lo commonly $2200, or $0300 or ??
	
;OS65D format
; trk1-n [8E1]<index><delay>$43,$57,bcd trk#,$58 <delay>$76,Sect#,SectLen,#len-pages-data,$47,$53<delay + more sectors>
; tracks can have 0 to 13 sectors, each sector can be 1 to 14 pages long plus overhead

;OS65U format
; trk1-n [8E1]<index><delay>$80,trkHi(0), trkLo,<delay>[8N1]$1F?,$E00 data + 2 bytes chksum + 2bytes ??

;HexDos format
; trk1-n [8E1]<index><delay>$57,trk#(binary),$896 bytes data, ends with $FF,$FF however only $800 bytes valid and 
; are duplicated on next track. On trk 0, the $96 bytes can contain short autostart program
; on format, track 0 copied, track1 has empty directory list (BASIC encoded), rest of tracks have header & 0s

;Dos/65 format (same as OS65D but all tracks have 1 large sector, including trk0 (doesn't boot from this disk)
; trk1-n [8E1]<index><delay>$43,$57,bcd trk#,$58,<delay?>, $76,$01,$08 (or $D on 8"),8(or13)*256 sector data

;PicoDOS
; same as OS65D one 8 page sector/track 5.25"

;OSI CP/M V1.4 & OSI CP/M 2.22 format
; trk1-76 8" only : 13 page sector encoded in OS65D format

;UCSD p-System
; trk1-n  OS65D format 5.25" = 8 page single sector per track. 8" = (12??) page single sector per track



; OSI HARDWARE
;  OSI 505 board / OSI 470 board
;  C000-C003 = PIA (decoded to C000-C00F) (C000/C040/C080/C0C0)
;  C010-C011 = ACIA (Decode to C010-C01F) (C010/C050/C090/C0D0)
;  C020-C03F = NULLREAD (C020/C060/C0A0/C0E0)

;  C000 = PORT A or DDR A
;  C001 = Control Reg A
;  C002 = PORT B or DDR B
;  C003 = Control Reg B
;  C010 = Serial data
;  C011 = Serial Control

; pin assignments
;  -> = output
;  $C000 (DDPIA)
;  PA0 <- !Drive 1 Ready		(0 = drive 1 ready)
;  PA1 <- !TRACK 00				(0 = head on trk 0)
;  PA2 <- !FAULT (nc)
;  PA3 <- !SECTOR (nc)
;  PA4 <- !Drive 2 Ready		(0 = drive 2 ready)
;  PA5 <- !Write Protect		(0 = write protect)
;  PA6 -> Drive Select			(0 = drv 2, 1 = drv 1)
;  PA7 <- !INDEX
; 
;  $C002
;  PB0 -> !WRITE ENABLE			(0 = write)
;  PB1 -> !ERASE ENABLE (nc)	(leave 1)
;  PB2 -> !STEP IN				(0 = to trk 39)
;  PB3 -> !RESET STEP 		    (step on 1->0 transition)
;  PB4 -> !FAULT RESET (nc)		(leave 1) (motor on?)
;  PB5 -> Drive Enable			(1 = drive ON) (Or Drive/Side sel. 4 drive system 1=drive 0&1,0=drive 2&3 )
;  PB6 -> !LOW CURRENT (nc)		(leave 1)
;  PB7 -> !HEAD LOAD			(0 = head on disk)
; 
;  CA1 <- 20ms clk on 8"
;  CA2
;  CB1 <- 400 mSec clk for RTC operations (2.5x/sec) 5.25",	1000ms for 8"
;  CB2 
; 

; constants
MAXTRK5	=	40	; # TRACKS TO READ 5 1/4 disk
MAXPAG5	=	9	; # PAGES TO READ  5 1/4 disk
MAXTRK8 =   77	; # Tracks to read 8 disk
MAXPAG8 =   15	; # Pages to read 8 disk
FILLCH	=	$00	; FILL CHARACTER

; I/O locations
DDPIA	=	$C000	; Disk controller PIA
DDACIA  =   $C010   ; Disk Controller Serial Port
C2ACIA  =   $FC00   ; ACIA on C2/C4/C8/C3
C1ACIA  =   $F000   ; ACIA on C1

;Control Character Constants XMODEM etc.
CR		=	$0D
LF		=	$0A
SOH		=	$01		; start block marker
STK     =   $02     ; start of 1K block marker (STX is reserved)
EOT		=	$04		; end of text marker
ACK		=	$06		; good block acknowledged
NAK		=	$15		; bad block acknowledged
CAN		=	$18		; cancel ^X (not standard, not supported)
ESC		=	$1B		; ESC to exit

;xmodem status return values
WRONGBLK = $FD		;xmode recv wrong blk #
MISMATCH = $FC
ABORTRECV = $FE	    ;xmode recv abort recv

;zero page storage space
STORE   =   $10   ;zero page storage location
MACHINE =   STORE+00 ;1 - C1/C2/C3 flag 00=C2/4/8, $40=C3Ser, $80=C1
MAXTRK  =   STORE+01 ;1 - 40 for 5.25, 77 for 8, 255 for HD
MAXTRH  =   STORE+02 ;1 - high byte max track for HD, 0 for floppy
MAXPAG  =   STORE+03 ;1 - 9 for 5.25, 15 for 8, 16 for HD
ACIACTL =   STORE+04 ;1 - controls cassette/rs-232 ACIA divisor, format
DRVNUM  =   STORE+05 ;1 - current drive# 0-3
TRK     =   STORE+06 ;1 - current track # BIN
TRKBCD  =   STORE+07 ;1 - current track # BCD
SECTNUM =   STORE+08 ;1 - current sector
CNT	    =   STORE+09 ;2 - # bytes sent 
TMP     =   STORE+11 ;2
DEST    =   STORE+13 ;2 
PDEC8	=   STORE+15 ;2 word - hex to decimal source
PDEC9	=   STORE+17 ;1 - flag byte
PDEC10	=   STORE+18 ;3 - byte result hex to decimal
USEXMOD =   STORE+21 ;1 - use xmodem 0x80=CRC 0xC0=Checksum  else HEX
DSKTYP  =   STORE+22 ;1 - disktype 0x80 when 8" OS65U 0x40 when not, 0 when undetermined, 1=hexdos 2=boot
CURSEC  =   STORE+23 ;1 sector counter
DATABEG =   STORE+24 ;1 byte offset to start of trk data
VERIFY  =   STORE+25 ;1 byte flag VERIFY count
NUMPAGES=   STORE+26 ;1 byte

;log support
HIGHMEM =   STORE+27 ; 1 byte - top page of RAM
LOGMEM  =   STORE+28 ; 2 bytes log top
					 
;xmodem transfer protocol support
lastblk	=	STORE+30 ;1 - flag for last block
blkno	=	STORE+31 ;1 - block number 
Xerrcnt	=	STORE+32 ;1 - error counter 10 is the limit
crc		=	STORE+33 ;1	- CRC lo byte  (two byte variable)
crch	=	STORE+34 ;1	- CRC hi byte  
EOFP	=	STORE+35 ;2 - end of file address pointer+1 (noninclusive) (2 bytes)
EOFPH	=	STORE+36 ;-- hi byte  
SOFP    =   STORE+37 ;2 - start of file address pointer (usually BUFFER)
SOFPH   =   STORE+38 ;-- hi byte
EOBP    =   STORE+39 ;2 - End of Buffer
EOBPH   =   STORE+40 ; -- hi byte
Xretry	=	STORE+41 ;1 - time delay / retry counter 
Xretry2	=	STORE+42 ;1 - 2nd counter
lastdata =  STORE+43 ;1 - flag buffer of file
Xsavem   =  STORE+44 ;1 - saved index to resume filling send buffer


;more zero page storage
DRVACIA =   STORE+45 ; 1 byte storage for current serial word format for DISK ACIA 8E1/8N1
NUMPAGS =   STORE+46 ; 1 bytes
PTRSTOR =   STORE+47 ; 2 bytes
TDATA   =   STORE+49 ; 2 bytes track data ptr
RDATA   =   STORE+51 ; 2 bytes

;video screen driver support
VIDSRC	=	STORE+54 ; 2 bytes .WORD $D040 src row for scroll, current line
VIDDST	=   STORE+56 ; 2 bytes .WORD $D000 top row for scroll
VIDOFFSET =	STORE+58 ; 1 byte current cursor pos
VIDXLEN	=	STORE+59 ; 1 byte screen width (physical screen width)
VIDXMAX	=	STORE+60 ; 1 byte logical line len
OUTFLAG =   STORE+61 ; 1 byte flag, 0x80 = disable video output, 0x40 = disable serial output
WRITERR =   STORE+62 ; 1 byte count of verify errors

				 
Xbuff		=	96	 ; $60 to $E5 temp 133($85) bytes xmodem buffer 

ORG	= $0300	
				       
	*=ORG
	.EXE *        ;A65 emit OSI .lod start address operation
	SEI
	CLD
	
	LDA	$FE01     ;determine machine type
	BEQ	SERTYP    ;Is this serial system?                                                    
	LDA	#$00	
	BIT	$DF00	  ;okay check C1/C2-C4
	BMI	C1TYPE
C1TYPE=*+1
	BIT	$80A9	  ;
SERTYP=*+1
	BIT	$40A9
	STA	MACHINE	  ;bit 7 = C1, bit 6 = C3, none = C2/C4
	LDA	#$15      ;B1 = 8N2 /16 RTStxIRQ  rxIRQ ;$15 = 8N1 /16 RTSNOtxIRQ NOrxIRQ; $B5 = irqs on 8N1
	STA	ACIACTL
	JSR	INZACIA
	JSR VIDINIT
	JSR TOPMEM	;locate top of memory for log messages, future disk buffer
	JSR SCRNCLR
	JSR LOGRESET
	JSR PRINT
	.BYTE $D,$A,'MemTop=$',0
	LDA HIGHMEM
	JSR PHEXA
	JSR PRINT
	.BYTE 'FF',$A,$D,0
	LDX	#$00
TOPMSG	
	LDA	SCRNMSG,X	;display 'ON SERIAL' message on screen for video based systems
	BEQ	TOPMSG1
	STA	$D0C5,X		;visible on C1 & C4
	INX
	BNE	TOPMSG
TOPMSG1	
	LDA #$C0
	STA	USEXMOD
	LDA	#$00
	STA	DRVNUM
	STA DSKTYP
	STA VERIFY
    BIT MACHINE
	BVC *+5
	JMP SETDRV8  ;serial systems use 8" by default
	JMP SETDRV5
	
; draw menu wait for selection to begin operation
TOP
	JSR VIDOFF
	JSR	DRWMENU

MENUL0
	JSR	INKEY		;get menu selection
	CMP #$20
	BCC MENUL0
	CMP #$7F
	BCS MENUL0
	JSR	OUTPUT		;echo input
    CMP #$60
    BCC *+4
    AND #$DF		;MakeUpper
	CMP	#'1
	BNE	*+5
	JMP	RAWREAD
	;CMP	#'2
	;BNE	*+5
	;JMP DOSREAD 
    CMP #'3
	BNE *+5
	JMP	XWRITE
	CMP	#'4
	BNE	*+5
	JMP	SETDRV
	CMP	#'5
	BNE	*+5
	JMP	DRVTYPE
	CMP	#'6
	BNE	*+5
	JMP	ACIADIV
	CMP	#'7
	BNE	*+5
	JMP	SETXFER
	CMP	#'8
	BNE	*+5
	JMP	($FFFC)	;Exit
	CMP #'L
	BNE MENUL1
	JSR LOGPRINT
	JSR ANYKEY
	JMP TOP
MENUL1
	LDA #'?
	JSR OUTPUT
	JMP TOP

DRVTYPE
	JSR	PRINT
	.BYTE $D,$A,'Enter your disk drive type (8) inch or (5).25 inch? >',0
	JSR	INKEY
	JSR	OUTPUT
	CMP	#$38
	BEQ	SETDRV8
	CMP	#$35
	BEQ	SETDRV5
INPERR
	LDA	#'?
	JSR	OUTPUT
	JMP	TOP

SETDRV5
	LDA	#MAXTRK5
	STA	MAXTRK
	LDA	#MAXPAG5
	STA	MAXPAG
	LDA #$40    ;flag not OS65U (it's a 5" drive)
	STA DSKTYP
	JMP	TOP

SETDRV8
	LDA	#MAXTRK8
	STA	MAXTRK
	LDA	#MAXPAG8
	STA	MAXPAG
	LDA #$00   ;flag disk type unknown (could be OS65U)
	STA DSKTYP	
	JMP	TOP


ACIADIV
	JSR PRINT
	.BYTE $D,$A,'Set (N)ormal or 16x (F)aster? >',0
	JSR INKEY
	JSR OUTPUT
	ORA #$20
	CMP #'f
	BEQ DIV1
	CMP #'n
	BEQ DIV16
	JMP INPERR

DIV1
	LDA	#$14  ;8N1 div 1

DIV16=*+1
	BIT	$15A9  ;$15A9	 ;15=8N1 /16 -RTS active no irq
	STA	ACIACTL
	JSR	INZACIA
	JMP	TOP

SETDRV
	JSR PRINT
	.BYTE $D,$A,'Select drive (A), (B), (C), (D) ? >',0
	JSR INKEY
	JSR OUTPUT
	AND #$DF
	CMP #'A
	BCC SETERR
	CMP #'E
	BCS SETERR
	SBC #'@		;carry is clear so val -1
	STA DRVNUM
	JMP TOP
SETERR
	JMP	INPERR

SETXFER
	JSR PRINT
	.BYTE $D,$A,'Use (X)modem, Xmodem(C)RC or(H)ex transfer? ? >',0
	JSR INKEY
	JSR OUTPUT
	AND #$DF
	CMP #'X
	BNE SETXF1
	LDA #$C0	;bit 7,6 set = Xmodem(Checksum)
	BNE SETXM
SETXF1
	CMP #'C
	BNE SETXF2
	LDA #$80
	BNE SETXM	;bit 7 set = Xmodem/CRC
SETXF2
	CMP #'H
	BNE SETERR ;hex
	LDA #$00
SETXM
	STA USEXMOD
	JMP TOP

;read raw data from disk
RAWREAD
	JSR LOGRESET
	JSR	PRINT
	.BYTE	CR,LF
	.BYTE	'WRITE PROTECT ALL DISKS!',CR,LF
	.BYTE	CR,LF
	.BYTE	'Insert disk to be dumped',CR,LF,0
	JSR ABORTMSG
	JSR	ANYKEY

	JSR	INITPIA
	JSR	SELDRV

	LDA	#0
	STA	lastdata

	JSR	TZERO	; STEP TO TRACK 0
	BCS	NOTRK
	LDA	#$58    ;0 10 110 00 ;$58-  recv irq, RTS HIGH no xmit irq,  8E1,  DIVIDE BY 1,
	STA	DRVACIA	

LOOP	
	LDA	TRK
	BNE	LOOP3
	JSR READMSG
	JSR	RDTZERO	;read with index hole timout check
	BCC	LOOP5	
	JMP	NODRIVE
LOOP3
	LDA	TRK	
	CMP	MAXTRK
	BCS	QUIT
	JSR READMSG
	JSR	RDTRKU
	BCC	LOOP5 
	JMP	NODRIVE
LOOP5
	;removed code
	
CHKRETN
	;automatic LDA	MAXPAG   ;how many pages to read?  09/0F
	JSR	SAVBUF
	BCS	QUIT
	JSR	FRACIANW
	BCC	LOOP4
	CMP	#$1B   ;ESC key?
	BEQ	QUIT
	CMP	#$18   ;^X
	BEQ	QUIT
LOOP4
	LDA	TRK
	CMP	MAXTRK
	BEQ	QUIT
	JSR	TNEXT	; STEP TO NEXT TRACK
LOOP2
	JMP	LOOP

QUIT
	JSR	TZERO
	JSR	INITPIA
	JSR	PRINT
	.BYTE	CR,LF,CR,LF
	.BYTE	';DONE ... REMOVE DISK',CR,LF,0
	JSR	INKEY
	JMP	TOP


NOTRK
	JSR	INITPIA
	JSR	PRINT
	.BYTE CR,LF,CR,LF, 'ERROR Seeking Track 0',0
	JSR	ANYKEY
	JMP	TOP	

NODRIVE
	JSR	TZERO
	JSR	INITPIA
	JSR	PRINT
	.BYTE CR,LF, 'ERROR Waiting for disk index!',0
	JSR	ANYKEY
	JMP	TOP

ABORTMSG
	JSR PRINT
	.BYTE CR,LF, 'ESC KEY OR ^X ABORTS',CR,LF,0
	RTS

READERR
	TYA
	PHA
	JSR PRINT
	.BYTE CR,LF, 'Read Error offset $',0
	PLA
	JSR PHEX
	JMP CRLF

; SAVE BUFFER CONTENTS (A) AS HEX

SAVBUF	
	JSR	RESPTR
	;automatic STA	NUMPAGS
	LDA	#0
	STA	CNT
	STA	CNT+1
	TAY
	TAX
	LDA	USEXMOD
	BNE	SAVXMOD1
	BEQ	SAVBUF5	 ;always

SAVBUF1	
	CPX	#0
	BNE	SAVBUF2
	JSR	CRLF

	JSR	FRACIANW
	BCC	SAVBUFX
	CMP	#$1B   ;ESC key?
	BEQ	SAVABORT
	CMP	#$18   ;^X
	BNE	SAVBUFX
SAVABORT	
	JSR PRINT
	.BYTE $A,$D,'Abort!',0
	SEC
	RTS


SAVBUFX
	LDX	#16
	BNE	SAVBUF3

SAVBUF2	
	LDA	#' '
	JSR	OUTPUT
SAVBUF3	
	LDA	(TDATA),Y
	JSR	PHEXA
	INC	CNT
	BNE	SAVBUF4
	INC	CNT+1
SAVBUF4	
	DEX
	INY
	BNE	SAVBUF1
	INC	TDATA+1
	DEC	NUMPAGS
SAVBUF5	
	LDA	NUMPAGS
	BNE	SAVBUF1
	JSR	CRLF
	CLC
	RTS


; SAVE BUFFER CONTENTS USING XMODEM

;SAVXMOD
;	JSR	RESPTR
;	STA NUMPAGS

SAVXMOD1

	LDA TDATA		;save the whole track buffer regardless of max read
	STA EOFP
	LDA TDATA+1
	CLC
	ADC MAXPAG
	STA EOFPH

	LDA	TRK
	CLC
	ADC #$01
	CMP	MAXTRK
	BNE SAVXMOD2
	LDA #$C0	 ;last block flag
	STA lastdata

SAVXMOD2
	JSR XResume

	BCC	XMODBUF5
	JSR PRINT
	.BYTE $A,$D,'Error sending data!',0
	SEC
	RTS

XMODBUF5	
	RTS
XQUIT
	JSR VIDOFF
	JMP TOP
XWRITE
	JSR LOGRESET
	JSR VIDON
	JSR	PRINT
	.BYTE	CR,LF
	.BYTE   CR,LF, 'Send Disk via XMODEM',CR,LF
	.BYTE   'Insert WRITABLE disk in drive ',0
    LDA DRVNUM
	CLC
	ADC #$41
	JSR	OUTPUT
	JSR PRINT 
	.BYTE   CR,LF,'ALL DATA WILL BE LOST!',CR,LF,0
	JSR ABORTMSG
	JSR	ANYKEY
	CMP	#$1B   ;ESC key
	BEQ	XQUIT
	CMP	#$18   ;^X
	BEQ	XQUIT
    
	JSR INIT
	JSR	INITPIA
	JSR	SELDRV
	LDA	#0
	STA	TRK
	STA TRKBCD
	STA	lastdata

	JSR	TZERO	; STEP TO TRACK 0
	BCC *+5	
	JMP NOTRK
	JSR CHKWP
	BNE XWRIT1
	JSR PRINT
	.BYTE CR,LF,LF,'ERROR, DISK WRITE PROTECTED',0
	JSR ANYKEY
	JMP XQUIT
XWRIT1	
	LDA	#$58    ;0 10 110 00 ;$58-  recv irq, RTS HIGH no xmit irq,  8E1,  DIVIDE BY 1,
	STA	DRVACIA	

		;prepare xmodem receive
		;when track buf full, write track
		;when disk is full , end
		;when transmission ends, end
	JSR XRcvDisk	;transfer a track via xmodem
	JMP QUIT

;if EOFP is at end of data then
;start at $EOFP-1 until $EOFP == $TDATA if contents of (EOFP) != 0 bail, exit pointing to next character
;
TRIMTRAK
	;count back from eofp to 1st non-zero greater than TDATA
	LDA EOFPH
	CMP EOBPH	;are we already less than End of Buffer?
	BCC TRM0
	BNE TRM6
	LDA EOFP
	CMP EOBP
	BCS TRM6	; EOFP >= EOBP	
TRM0			
	SEC			;Already set
	RTS
TRM6
	LDA EOBPH
	STA EOFPH
	LDA EOBP
	STA EOFP
	LDY #$00
	BEQ TRM5	;count down 1st byte EOFP is 1 higher than end of data
TRM1
	LDA (EOFP),Y
	BNE TRM4
TRM5
	LDA EOFP	;subtract 1 from EOFP
	SEC
	SBC #$01
	STA EOFP
	BCS TRM3
	DEC EOFPH
TRM3
	LDA TDATA+1	;compare EOFP to current top of data
	CMP EOFPH
	BCC TRM1
    BNE TRM4	
	LDA TDATA
	CMP EOFP	; Tdata < EOFP?
	BCC TRM1	 
				;no tdata >= EOFP
TRM4			
	INC EOFP	;move 1 higher
	BNE TRM2
	INC EOFPH
TRM2
	CLC
	RTS



;callback from xmodem receive	
XWriteTrk=*
	LDA TRK
	CMP MAXTRK
	BCC XWT0
    ; too much data sent ... ignore	
	JSR PRINT
	.BYTE CAN,CAN,CAN,$A,$D,'Disk capacity exceeded.',0
	JMP DONE
XWT0
	JSR SEROFF
	JSR GETEOT		;set track end based on stored value sent in data block
	JSR CHKFMT    	;X will contain offset to start of track for recognized formats // (now always 0)
	LDX #$00
	STX DATABEG
	JSR TRIMTRAK    ;could be trimming valid data but don't want to write track of fill characters from previous read
	BCS XWTE
	JSR SETEOT		;track len changed
XWTE
	JSR CRLF
XWT7	
	JSR PRINT
	.BYTE 'Write Track ',0
	LDA TRK
    JSR PDECA
	JSR PRNFMT
	BIT DSKTYP
	BVC xwt3
	LDA #$20	  ;print 65d sector list	
	JSR OUTPUT

	LDX #$01
	LDA NUMSECTS  ;0 to 13
	STA CNT
	BNE xwt4
	JSR PRINT
	.BYTE '0 ',0
	JMP XWT3
XWT4             ;print 65d sector list
	TXA
	PHA
	JSR PDECA
	LDA #'/
	JSR OUTPUT
	PLA
	PHA
	TAX
	LDA SectLenH-1,X
	JSR PDECA
	LDA #$20
	JSR OUTPUT
	PLA
	TAX
	INX
	DEC CNT
	BNE XWT4

XWT3
	LDA SOFP
	STA TDATA
	LDA SOFPH
	STA TDATA+1
	JSR GETEOT
	JSR PDEC
	
	JSR WRNEXT
	;verify

	INC VERIFY
	JSR INIT
	JSR RDNOLOAD	;rd trk to verify buffer
	JSR RESVER		;reset verify pointers
	JSR CHKFMT
	JSR TRIMTRAK
	JSR HIDESIZ		;skip EOT size
	JSR CMPTRK
	BEQ XWT6
	INC WRITERR
	JSR PRINT
	.BYTE '...error', 0
	LDA VERIFY
	CMP #$03
	BCS XWT8
	JSR PRINT
	.BYTE $D,$A, 'RETRY ', 0	
	JSR RESPTR	;prepare for write again
	JSR GETEOT
	JSR CHKFMT  ;recreate sector list	
    JMP XWT7
	BCS XWT8 
XWT8   	
	JSR PRINT
	.BYTE ' ** Failed **',0
	JMP XWT9

XWT6
	JSR PRINT
	.BYTE '...OK',0	
XWT9
	LDA #$00
	STA VERIFY	
	JSR UNLOAD ;done with this track
	
	LDA TRK
	CMP MAXTRK
	BCS XWT5
	JSR TNEXT
XWT5
	JSR	RESPTR	; SET POINTERS
	JSR SERON
	JMP XNextBlk

; INIT POINTERS, CLEAR BUFFER
INIT
	LDA VERIFY
	BEQ INIT2
	JSR RESVER ;prepare for verify
	BCC INIT3
INIT2
	JSR RESPTR
INIT3
	LDY	#0
	LDA	#FILLCH
INIT1			;ERASE TRK BUFFER
	STA	(TDATA),Y
	INY
	BNE	INIT1
	INC	TDATA+1
	DEC	NUMPAGS
	BNE	INIT1
	;fall through to re-fix buffer start

RESCUR	;reset pointers based on current mode read or verify
	LDA VERIFY
	BNE RESVER
	
; RESET BUFFER POINTERS
RESPTR
	PHA
	LDA	MAXPAG   ;how many pages of track data?  09/0F
	STA	NUMPAGS
	LDA	#>BUFFER
	STA	TDATA+1
	STA SOFP+1
	CLC
	ADC MAXPAG
	STA EOFPH
	STA EOBPH
	DEC EOBPH	
	LDA	#<BUFFER
	STA	TDATA
	STA EOFP
	STA SOFP
	LDA #$FC
	STA EOBP

	PLA
	SEC
	RTS
	

	
; RESET BUFFER POINTERS FOR VERIFY BUFFER
RESVER
	PHA
	LDA	MAXPAG   ;how many pages of track data?  09/0F
	STA	NUMPAGS
	LDA	#>VBUFF
	STA	TDATA+1
	STA SOFP+1
	CLC
	ADC MAXPAG
	STA EOFPH
	STA EOBPH
	DEC EOBPH
	LDA	#<VBUFF
	STA	TDATA
	STA SOFP
	STA EOFP
	LDA #$FC
	STA EOBP
	PLA
	CLC
	RTS
	
HIDESIZ			;drop EOFP by 4 bytes to protect #bytes read at end of track buffer
	LDA SOFPH
	CLC
	ADC MAXPAG
	CMP EOFPH
	BCC HIDESIZ-1 ;its already lowered
	LDA EOFP
	SEC
	SBC #$04
	STA EOFP
	LDA EOFP+1
	SBC #$00
	STA EOFP+1
	RTS
	
CMPTRK		;Compare track data from start to EOFP.  EOFP points to position in Verify Buffer
			;EOFP should be 1 higher than end of data
	LDA	#<BUFFER
	STA	TDATA
	LDA	#>BUFFER
	STA	TDATA+1	
	LDA #<VBUFF
	STA RDATA
	LDA #>VBUFF
	STA RDATA+1
	LDY #$00
	BEQ CMPTR5
	
CMPTR3
	LDA (TDATA),Y
	CMP (RDATA),Y
	BNE CMPTR1
	INC RDATA
	BNE CMPTR4
	INC RDATA+1
CMPTR4
	INC TDATA
	BNE CMPTR5
	INC TDATA+1
CMPTR5
	LDA RDATA+1
	CMP EOFP+1
	BCC CMPTR3
	LDA RDATA
	CMP EOFP
	BCC CMPTR3
	CLC
	RTS
CMPTR1
	SEC
	RTS

; INIT DISK CONTROLLER PIA
INITPIA	
	LDY	#0
	LDA	#$40
	STY	DDPIA+1	; SELECT DDRA
	STA	DDPIA	; SET PORTA TO ALL INPUTS except PA6
	LDX	#4		; 0000 0100
	STX	DDPIA+1	; SELECT PORTA
	STA	DDPIA   ; SET PB6 HIGH
	STY	DDPIA+3	; SELECT DDRB
	DEY
	STY	DDPIA+2	; SET PORTB TO ALL OUTPUTS
	STX	DDPIA+3	; SELECT PORTB
	STY	DDPIA+2	; SET PORTB OUTPUTS HIGH
	RTS


;SELECT DRIVE PB5 PA6  DRIVE (1-4)
;              0   0	#4
;              0   1	#3
;              1   0	#2
;              1   1	#1

SELDRV
	LDA	DRVNUM
SELADRV
	LSR	A
	TAY
	BCC	*+5
	LDA	#$00
	BIT	$40A9	; This is LDA #$40 if BCC branches
	STA	DDPIA	; This is A/B select in port A
	LDA	DDPIA+2
	ORA	#$20
	CPY	#$01
	BCC	*+4
	AND	#$DF
	STA	DDPIA+2	; This is master select in port B
	RTS	
	
CHKWP    ;check write protect 1=writable 0=protected
	LDA DDPIA  ;Test PA5
	AND #$20
	RTS

; STEP TO TRACK 0
TZERO
	LDA	MAXTRK	;max num tracks
	ADC	#$08	;plus a few more
	STA	TMP		;MAX times to step before abort
    JSR TNEXT

TZERO1	
	LDA	#2		; 0000 0010
	BIT	DDPIA	; TEST 'TRK0' PIN
	BNE	TZERO2	
	LDA	#0		; AT TRACK 0
	STA	TRK
	STA TRKBCD
	CLC
	RTS

TZERO2	
	JSR TPREV
	DEC	TMP
	BNE	TZERO1
	SEC			;still not at 0 after all steps (abort)
	RTS	

; STEP TO PREVIOUS TRACK
TPREV
	LDA DDPIA+2 
	ORA #$04	; DIR=(TO TRK0)
	DEC	TRK
	PHP
	SEI
	PHA
	SED
	LDA TRKBCD
	SEC
	SBC #$01
	STA TRKBCD
	CLD
	PLA
	PLP
	JMP TSTEP
	
; STEP TO NEXT TRACK
TNEXT	
	LDA DDPIA+2
	AND	#$FB	; 1111 1011   DIR=INWARDS
	INC	TRK
	PHP
	SEI
	PHA
	SED
	LDA TRKBCD
	CLC
	ADC #$01
	STA TRKBCD
	CLD
	PLA
	PLP
TSTEP 	;common to tnext & tprev
	STA	DDPIA+2	; SET 'DIR' PIN
	JSR	DELAY2
	AND	#$F7	; 1111 0111
	STA	DDPIA+2	; SET 'STEP' LOW
	JSR	DELAY2
	ORA	#8		; 0000 1000
	STA	DDPIA+2	; SET 'STEP' HIGH
	; Set reduced write current (used on some 8" drives)
REDWR
	LDA	TRK
	CMP	#43		; Is track >= 43
	LDA	DDPIA+2
	AND	#$BF	; Set low current active
	BCS	SETLO	; Yes
	ORA	#$40	; Set low current inactive
SETLO
	STA	DDPIA+2
	LDX	#32	    ; 40MS
	JMP	DELAY

; Load Head of current drive
LOADHEAD
	LDA DDPIA+2  ; PRESERVE DISK SEL BIT
	AND	#$7F	; 0111 1111
LHEAD1
	STA	DDPIA+2	; LOAD DISK HEAD
	LDX	#$FF	; 320MS
	JSR	DELAY
	DEX
	JMP DELAY    ; 320MS again in case loadhead is also motor on

;Unload head of current drive
UNLOAD
	LDA DDPIA+2
	ORA #$80	
	JSR LHEAD1
	CLC
	RTS
	
; READ CURRENT TRACK TO BUFFER
; RETURNS Y=index of last byte read
RDTZERO
	JSR	INIT
	JSR LOADHEAD
RDNOLOAD
	LDA #$20
	STA TMP
	LDX #$00
RDTRKY
    LDY #$40
RDTRK1
	BIT	DDPIA	; WAIT for INDEX PULSE with timeout
	BPL	RDTRK8
	DEX
	BNE RDTRK1
	DEY
	BNE RDTRK1
	DEC TMP
	BNE RDTRKY
	SEC
	RTS


RDTRK8
	LDX	DRVACIA	; usually $58 -DIVIDE BY 1, 8E1, RTS HIGH no IRQs
	LDA	#3
	LDY	#0		
RDTRK2			
	BIT	DDPIA	; WAIT END OF
	BPL	RDTRK2	; INDEX PULSE
	STA	DDACIA  ; RESET ACIA
	STX	DDACIA  ; SET DISK SERIAL WORD FORMAT/CLEAR FLAGS
				 
		; READ UNTIL ALL PAGES DONE
RDTRK3
	BIT	DDPIA	; TEST FOR INDEX PULSE PRESENT AGAIN
	BPL	RDTRK7
	LDA	DDACIA	;test byte ready?	
	LSR	A
	BCC	RDTRK3	;byte not ready?

RDTRK4
	LDA	DDACIA+1 ; READ BYTE
RDTRK9
	STA	(TDATA),Y
	INY
	BNE	RDTRK3
	INC	TDATA+1
	DEC	NUMPAGS
	BNE	RDTRK3
RDTRK7
	JMP RDTRKU5		;end & unload
	
; READ TRACK 1-76 TO BUFFER OS65U
; RETURNS Y=index of last byte read

RDTRKU
	JSR	INIT
	JSR LOADHEAD
    BIT	DDPIA
	BPL *-3     ;CHECK FOR INDEX ALREADY
RDTRKU1	
	BIT	DDPIA	; WAIT START OF	INDEX PULSE
	BMI	RDTRKU1
	PHA
	PLA
	PHA 
	PLA
	PHA   ;delay
	PLA
	LDX	DRVACIA	; usually $58-DIVIDE BY 1, 8E1, RTS HIGH no IRQs
	LDA	#3
	LDY	#0	
    BIT DSKTYP
    BVS RDTRK2  ;can't be OS65U, we don't look for it
	
RDTRKU2			; now past index hole, reset ACIA, prepare to read data
	BIT	DDPIA	; WAIT END OF
	BPL	RDTRKU2	; INDEX PULSE
	
	STA	DDACIA  ; RESET ACIA
	STX	DDACIA  ; SET DISK SERIAL WORD FORMAT/CLEAR FLAGS
	
	LDX #$00	
RDTRKUP
	; READ 1st 3 bytes as 8E1 OS65U uses $80, tkHi, tkLo  OS65D uses $43, $57,Trk#,$58
	BIT	DDPIA	; TEST FOR INDEX PULSE PRESENT AGAIN
	BPL	RDTRKU5
	LDA	DDACIA  ;test byte ready?	
	LSR	A
	BCC	RDTRKUP	 ;byte not ready?
	LDA	DDACIA+1 ; READ BYTE
	STA	(TDATA),Y
	CPX #$00
	BNE RDTYPE
	CMP #$43
	BNE *+3	  ;flag this is a 65D format 8E1 and continue reading there
	TAX
	CMP #$80
	BNE *+3
	TAX
RDTYPE
	INY
	CPY #$03
	BNE RDTRKUP
	BIT DSKTYP
	BMI RDELAY0  ;force OS65U read
	CPX #$80
	BNE RDTRK3   ;didn't find OS65U start of track marker so continue in current format

	; once we've read 1st 3 bytes at 8E1, reset ACIA & set 8N1
RDELAY0
	LDX #$07
RDELAY1
	DEX
	BNE RDELAY1	 ;delay same as 65u
	LDA #$03
	LDX #$54   ; ACIA DIVIDE BY 1, 8N1, RTS HIGH no IRQs
	STA	DDACIA ; RESET ACIA
	STX	DDACIA ; SET DISK SERIAL WORD FORMAT/CLEAR FLAGS

RDTRKU3		;transition to 8N1 & skip noise byte
	BIT	DDPIA	; TEST FOR INDEX PULSE PRESENT AGAIN
	BPL	RDUCHK
	LDA	DDACIA  ;test byte ready?	
	LSR	A
	BCC	RDTRKU3	 ;byte not ready?
	LDA	DDACIA+1 ; READ BYTE
	BMI RDTRKU3  ; skip transition byte(s) with hi bit set	(noise)
	BPL RDTRKU8

RDTRKU4		;finally read rest of track
	BIT	DDPIA	; TEST FOR INDEX PULSE PRESENT AGAIN
	BPL	RDUCHK
	LDA	DDACIA  ;test byte ready?	
	LSR	A
	BCC	RDTRKU4	 ;byte not ready?
	LDA	DDACIA+1 ; READ BYTE
RDTRKU8
	STA	(TDATA),Y
	INY
	BNE	RDTRKU4
	INC	TDATA+1
	DEC	NUMPAGS
	BNE	RDTRKU4

RDUCHK
.IFN 0 <
	LDA TRK
	BEQ *+5
	JSR OSUCHK
    BCC RDTRKU5
	JSR CHKERR
>
RDTRKU5			 ;now at end of track/index pulse start
	TYA
	CLC
	ADC TDATA
	STA EOFP
	LDA TDATA+1
	ADC #$00
	STA EOFPH
	JSR SETEOT
	JMP UNLOAD

; Set end of track length at last 2 bytes of track buffer $XXFE
; Store XOR of value in 2nd to last 4 bytes  $XXFC
; on entry, EOFP contains high address
; to be used when recreating disk image
SETEOT		
			
	LDA SOFP+1	
	CLC
	ADC MAXPAG		;calculate buffer target address
	STA PTRSTOR+1
	LDA SOFP		;we know buffer is page aligned, this is 0 but just in case...
	STA PTRSTOR
	DEC PTRSTOR+1	;previous page

	LDY #$FE		;end of buffer
	CLC
	LDA EOFP		;add Y to buffer pos low, (should be 0)
	STA (PTRSTOR),Y	;store low value
	EOR #$FF
	TAX
	LDA EOFP+1		; last byte position hi
	ADC #$00
	SEC
	SBC SOFP+1		;remove buffer offset
	INY
	STA (PTRSTOR),Y	;store hi value
	EOR #$FF
	DEY
	DEY
	STA (PTRSTOR),Y	;store inverse hi
	TXA 
	DEY
	STA (PTRSTOR),Y ;store inverse lo
	RTS

;set eofp based on size stored at end of buffer	
GETEOT
	LDA SOFP+1	
	CLC
	ADC MAXPAG		;calculate buffer target address
	STA PTRSTOR+1
	LDA SOFP		;we know buffer is page aligned, this is 0 but just in case...
	STA PTRSTOR
	DEC PTRSTOR+1	;previous page

	LDY #$FC
	LDA (PTRSTOR),Y  ;XOR Lo
	EOR #$FF
	TAX
	INY
	LDA (PTRSTOR),Y  ;XOR Hi
	EOR #$FF
	PHA
	INY
	INY
	CMP (PTRSTOR),Y ;hi val
	BNE GETEOT1
	CMP MAXPAG
	BCS GETEOT1	;if value > track size its invalid
	TXA
	DEY
	CMP (PTRSTOR),Y ;lo val
	BNE GETEOT1
	CLC
	ADC SOFP
	STA EOFP
	TXA
	TAY
	PLA
	PHA
	ADC SOFPH
	STA EOFPH
	PLA
	CLC
	RTS
GETEOT1
	TXA
	TAY
	PLA
	SEC
	RTS
	

	
OSUCHK    ;check OS65U checksum  3 byte track header + $E00 data bytes
    JSR RESCUR
	LDA #$04
	STA PTRSTOR
	LDA #$0F    ;1 higher than target #pages  OS65U sector is always $E00 + overhead
	STA PTRSTOR+1
    LDA #$00	
	TAX
	TAY
OSCHK1
    CLC
	ADC (TDATA),y
	BCC OSCHK2
	INX
OSCHK2
    INY
	BNE OSCHK3
	INC TDATA+1
OSCHK3
	DEC PTRSTOR
	BNE OSCHK1
	DEC PTRSTOR+1
	BNE OSCHK1
	INY
	STA crc
	CMP (TDATA),y
	BNE OSCHKF
	DEY
	TXA
	STA crch
	CMP (TDATA),Y
	BNE OSCHKF
OSCHKOK=*
	CLC
	RTS
OSCHKF
	SEC
	RTS
	
; DELAY 1.25 MS PER LOOP at 1 MHz clock. X,Y modified
; X = # of loops
DELAY	
	LDY	#$F8
DELAY1	
	DEY
	BNE	DELAY1
	EOR	$FF,X
	DEX
	BNE	DELAY
	RTS

;delay 1600us * A   X,Y modified	
; exit on timeout = carry clear, or immediate exit on disk index set, carry set
DELAY16
	NOP		;$EA 1MHz  ;$0A 2MHz
	TAX

DELAY4
	LDY #$8F
DELAY5
	BIT	DDPIA
	BPL DEXIT
	DEY
	BNE DELAY5
	DEX
	BNE DELAY4
	CLC
	RTS
DEXIT
	SEC
	RTS

DELAY2	
	JSR	DELAY21
DELAY21	
	RTS

NUMSECTS 	;number of sectors in list
	.byte 0
	
;storage for up to 16 sector offsets in track data	
SectLenL
	.byte 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
	
	.byte $4 ;previous sector length for 1st sector
SectLenH
	.byte 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
	;32 byte sector size storage for 15 sectors
	
SectPozL
	.byte 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
SectPozH
	.byte 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
	;32 byte sector lenghts
	

TRK065
.BYTE $43,$57,0,$58

;DSKTYP = 0x01 when hexdos, 0x02 when boot, 0x80 when 8" reading OS65U, 0x40 when OS65D, 0 when undetermined
CHKHEXD		;check for hexdos disk type [$57], trk# (binary)
	INY
	LDA (SOFP),Y
	CMP TRK			;binary track #
	BNE CHKFM2
    LDA #$01
	STA DSKTYP	;flag hexdos fmt
	RTS
CHKZERO			;check for valid track 0 (boot) format ...loadHi,loadLo,#pages,
	LDA TRK065,Y
	CMP (SOFP),Y	;Dos-65 writes OS65D format on trk 0, check for it first
	BNE CHKZ1
	INY
	CPY #$04
	BNE CHKZERO
	DEY
	JMP CHK65D1
CHKZ1 
	LDY #$00
	LDA #$F0
	CMP	(SOFP),Y	;target page cant be > F000 normally, usually 2200 or 0300 or E0XX?
	BCC CHKFMT1-1
	LDY #$02
	LDA MAXPAG
	CMP (SOFP),Y	;#pages on disk can't be more than maxpag for this disk type --usually 8 or 9 (5.25"), 12 - 14 (8") 
	BCC CHKFMT1-1
	LDA (SOFP),Y	; Set current end of data ptr
	CLC
	ADC SOFPH
	STA TDATA+1
	LDA #$03
	STA TDATA
CHKZER
	LDA #$02		; flag boot fmt 
	STA DSKTYP	
	RTS
;
; Main entry to check disk buffer against known formats
; if it matches, disk buffer noise is removed
; returns with DSKTYP set and X set to offset to start of track data
;	
CHKFMT			;check track data for OS65U, OS65D, hexdos formats  
	LDY #$00
	LDX #$00
	STY DSKTYP	;undetermined
	STY SectLenH
	LDA SOFP	;set pointers to start at begining of track data
	STA CHKVAL
	LDA SOFP+1
	STA CHKVAL+1
	LDA TRK
	BEQ CHKZERO	;boot track usually or hexdos data track maybe
	DEX	
CHKFMT1
	INX
	TXA
	TAY
CHKVAL= *+1
	LDA	BUFFER,X	;set above to start of file pointer (BUFFER or VBUFF)
	CMP #$80
	BEQ CHK65U		;could be OS65U
	CMP #$43
	BEQ CHK65D		;could be OS65D
	CMP #$57
	BEQ CHKHEXD		;could be hexdos
CHKFM2			;return from type check if not matched
	CPX #$12
	BCC CHKFMT1		;how many bytes to test before giving up?
	LDX #$00
	RTS		;unknown format?

CHK65U ;check for [$80],$00,$trk,$58
	INY
	LDA MAXPAG
	CMP #MAXPAG8
	BNE CHKFMT1		;no OS65U on 5.25"
	LDA	(SOFP),Y
	BNE CHKFM2		;chk hi trk# is 0
	INY				
	LDA	(SOFP),Y	
	CMP trk         
	BNE CHKFM2
	LDA #$80		;flag OS65U fmt
	STA DSKTYP
	JSR U65CHECK    ;fix trk data noise
	RTS

CHK65D   ; [$43], $57, BCD TRK No (01-39, 01-77), $58
	INY
	LDA	(SOFP),Y
	CMP #$57
	BNE CHKFM2
	INY
	LDA	(SOFP),Y
	CMP #$77      ;check for out of range track not real track
	BCS CHKFM2
	INY				
	LDA	(SOFP),Y
	CMP #$58
	BNE CHKFM2
CHK65D1
	LDA #$40
	STA DSKTYP	;flag os65D fmt
	LDA	SOFP
	STA	TDATA
	LDA	SOFP+1
	STA	TDATA+1
	TXA
	TAY
	LDX #$00
	JSR MOVBLK  ;delete possible start of track noise	

	JSR SECTCOUNT	;count sectors & identify offsets
	RTS

; Remove noise before start of trak header OS65U
; Remove possible noise between track header and start of sector data
; noise is identified by hi bit being set.	
U65CHECK   ;x=offset to data start
	LDA	SOFP  ;reset some pointers
	STA	TDATA
	LDA	SOFP+1
	STA	TDATA+1
	TXA
	TAY
	LDX #$00
	JSR MOVBLK  ;delete possible start of track noise
	LDA #$01
	STA NUMSECTS
	;after header, there may be a noise byte with high bit set otherwise its start of data
U65C1
	LDY #$03		;skip track header
	LDA (TDATA),Y
	BPL U65C2
	LDX #$03
	INY
	JSR MOVBLK   ;delete noise byte
	BCC U65C1
U65C2
	JSR OSUCHK	;check OS65U track for Checksum Error
	BCC U65C3
	LDA VERIFY
	BNE U65C3
	LDA (TDATA),Y ;data CRCH
	PHA
	DEY
	LDA (TDATA),Y
	PHA
	INY
	JSR PRINT
	.BYTE $D,$A,'Bad 65U checksum, should be $',0
	LDA crch
	LDX crc
	JSR PHEX
	JSR PRINT
	.BYTE ' not $',0
	PLA
	TAX
	PLA
	JSR PHEX
U65C3
	INY		;Pad rest of OS65U track data (noise) with 0
	INY		; 2 bytes after CRC
	TYA
	CLC
	ADC TDATA	; add Y to pointer to get current address
	STA TDATA
	STA EOFP
	LDA TDATA+1
	ADC #$00
	STA TDATA+1
	STA EOFPH
	LDY #$00
	BEQ U65C5
U65C4
	LDA #$00
	STA (TDATA),Y
	INC TDATA
	BNE U65C5
	INC TDATA+1
U65C5			;are we at EOB?
	LDA TDATA+1
	CMP EOBP+1
	BCC U65C4	;high byte less than target? continue
	LDA TDATA
	CMP EOBP
	BCC U65C4	;lo byte less than target? continue
	JSR SETEOT 
    LDX #$00
	RTS

	
SECTCOUNT	;determine number of 65D sectors and length $76,Sect#,SectLen,<SectLen*256 data>,[$47,$53]
	TXA
	PHA
	LDA #$04 ;skip track header
	CLC
	ADC SOFP
	STA RDATA
	LDA #$00	
	TAX
	ADC SOFP+1
	STA RDATA+1
	LDA #$01
	STA NUMSECTS
	BNE SECTL2
SECTL1
	LDY #$00
	LDA (RDATA),Y
	CMP #$76		;start of sector marker
	BEQ SECTST
SECTL3
	CMP #$00		;track filler? Count to end of data to finish
	BEQ SECTL4
	
	JSR DELBYTE		;delete garbage char before sector marker
	BCC SECTL2
SECTL4
	INC RDATA
	BNE SECTL2
	INC RDATA+1
SECTL2
	LDA RDATA+1
	CMP EOBP+1
	BCC SECTL1
	LDA RDATA
	CMP EOBP
	BCC SECTL1	
	; at end of track
	DEC NUMSECTS
	PLA
	TAX
	RTS

	
SECTST  ;found $76, look for expected sect#, X=sector index, Y=offset to sector data
	INY
	LDA (RDATA),Y  ;sect#
	CMP NUMSECTS
	BNE SECTL3     ;not found, continue search
	INY
	LDA (RDATA),Y   ;num pages
	CMP MAXPAG
	BCS SECTL3    ;bad number of pages in sector
	
	STA SectLenH,X
	INY ;index of byte after header
	TYA
	STA SectLenL,X  ;length includes sector header
	LDA RDATA
	STA SectPozL,X ;starting address including header
	STA TMP
	LDA RDATA+1
	STA SectPozH,X
	CLC
	ADC SectLenH,X
	STA TMP+1
	STA RDATA+1  ;offset to search for next start of sector

	LDA (TMP),Y
	CMP #$47		;skip over possible end of sector marker (not always present)
	BNE SECTST1
	INY
	LDA (TMP),Y
	CPY #$53
	BNE SECTST1
	INY
SECTST1
	TYA
	STA SectLenL,X
	INY	;Y points to char past end of sector
	TYA
	CLC
	ADC RDATA
	STA RDATA
	LDA RDATA+1
	ADC #$00
	STA RDATA+1
	INX
	INC NUMSECTS
	JMP SECTL2

;prepare to remove noise bytes from sector data
; Shift Sector data down by Y bytes
DELBYTE	
	TXA
	PHA
	TYA
	PHA
	LDA #$01
	STA Xsavem
	LDA RDATA+1
	STA TMP+1
	STA DEST+1
	LDA RDATA
	STA DEST
	CLC
	ADC Xsavem
	STA TMP
	BCC DELBYT1
	INC TMP+1
DELBYT1
	LDA EOFP
	SEC
	SBC RDATA
	STA CNT
	LDA EOFP+1
	SBC RDATA+1
	STA CNT+1
	LDY #$00
	JSR MOVBL2
DELEND    
	PLA
	TAY
	PLA
	TAX
	CLC
	RTS
	
;MOVBLK delete junk bytes from TDATA buffer Y=SRC offset, X=DEST within one page EOFP = End 
;on entry AAAAABBBBCCCCC  Y=10, X=6
;              ^   ^
;              X   Y
;on exit  AAAAACCCCC0000
;X<Y!
;use CNT for # bytes to move
;DEST as target
;TMP as src	
ENDMOV
	SEC
	RTS
MOVBLK
    STX TMP
	TYA
	SEC
	SBC TMP   ;Y-X

	STA Xsavem	;store diff
	BCC ENDMOV ; error exit X>=Y
	BEQ ENDMOV ; nothing to do
	CLC
	TYA
	ADC TDATA  ;add Y to TDATA to get SRC address
	STA TMP
	LDA TDATA+1
	ADC #$00
	STA TMP+1
	LDA EOFP  ;calculate # bytes to move
	SEC
	SBC TMP
	STA CNT
	LDA EOFP+1
	SBC TDATA+1
	STA CNT+1
	INC CNT   ;add 1 to count, its inclusive
	BNE MOVBL4
	INC CNT+1
MOVBL4	
	TXA		;calculate target address
	CLC
	ADC TDATA
	STA DEST
	LDA TDATA+1
	ADC #$00
	STA DEST+1
	LDY #$00
MOVBL2				;move bytes down
	LDA (TMP),Y
	STA (DEST),Y
	INY
	BNE MOVBL1
	INC TMP+1
	INC DEST+1
MOVBL1	
    DEC CNT
	BNE MOVBL2
	LDA CNT+1
	BEQ MOVBL5
	DEC CNT+1
	BPL MOVBL2
MOVBL5
	LDA #$00
	LDX Xsavem
    ;INX
MOVBL3				;fill space at end of block with 0's
	STA (DEST),Y
	INY
	BNE MOVBL6
	INC DEST+1
MOVBL6
	DEX
	BNE MOVBL3
	CLC
	RTS
	

;continue to write track, switch to OS65U format TDATA points to track data
WR65U 
	CLC
	LDA	#07    ;E04+trk header
	ADC TDATA
	STA EOFP
	LDA #$0E
	ADC TDATA+1
	STA EOFPH
	LDX #$03  ;3 bytes in OS65U header 

WR65UH 		;1st write track header in 8E1 fmt
	BIT	DDPIA	; WAIT 
	BMI	WR65UHOK	; Didn't find INDEX PULSE (good!)
	JMP	INDEXERR	;got index while trying to write data (error)
WR65UHOK
	LDA	#2
	AND	DDACIA		;test tx ready?	
	BEQ	WR65UH	    ;not ready?
	LDA	(TDATA),Y
	STA	DDACIA+1	; Write data to floppy
	INC TDATA
	DEX
	BNE	WR65UH	
	
UDELAY0
	LDX #$07
UDELAY1
	DEX
	BNE UDELAY1	 ;delay same as 65u (wait for byte to clock out?)
	LDA #$03
	LDX #$54   ; ACIA 8N1 /1 NORTS NOtxIRQ NOrxIRQ
	STA	DDACIA ; RESET ACIA
	STX	DDACIA ; SET DISK SERIAL WORD FORMAT/CLEAR FLAGS
;now write $E04 bytes in 8N1 FMT
	BNE WRDATALP ;always
	
;WRITE to current track the contents of TDATA. last byte is (eofp)	
;DSKTYP controls track format (OS65U/OS65D-like)
;DATABEG is offset to start of data (usually 0)
WRNEXT

	JSR LOADHEAD
	LDX DRVACIA	; prepare ACIA value for after reset usually $58-DIVIDE BY 1, 8E1, RTS HIGH no IRQs
	LDA	#3
	STA	DDACIA  ; RESET ACIA  turn off ACIA while waiting for index
	
	BIT	DDPIA	; WAIT FOR
	BMI	*-3	    ; INDEX PULSE
    
	;turn on write during index to keep glitch from happening after index
	LDA	DDPIA+2 ; PRESERVE DISK SEL BIT
	AND	#$FC	; TURN ON WRITE AND ERASE ENABLE
	STA	DDPIA+2

	BIT	DDPIA	; WAIT END OF
	BPL	*-3	; INDEX PULSE
	
	;now set ACIA parameters
	STX DDACIA  ; SET DISK SERIAL WORD FORMAT/CLEAR FLAGS

	LDX	#1	; 0.8MS
	LDY	#$9E
	JSR	DELAY1
	LDX #$04
	LDA	DATABEG ; usually 0 or offset to track header
	CLC
	ADC TDATA
	STA TDATA
	LDY #$00
	STY DATABEG
	BIT DSKTYP
	BVS WR65D	; OS65D format
    BMI WR65U	; OS65U format
	LDA SOFP
	STA TDATA
	LDA SOFPH
	STA TDATA+1
	LDY #$00
	;write track data as found w/o regard to formatting & delays
WRDATALP
	BIT	DDPIA	; WAIT 
	BPL	INDEXERR	;got index while trying to write data (error)
WNOINDEXERR
	LDA	#2
	AND	DDACIA		;test tx ready?	
	BEQ	WRDATALP	;not ready?
	LDA	(TDATA),Y
	STA	DDACIA+1	; Write data to floppy
	INC TDATA
	BNE	WRN1
	INC	TDATA+1
WRN1
	LDA TDATA+1
	CMP EOFPH
	BCC WRDATALP
	LDA TDATA
	CMP EOFP

	BCC	WNOINDEXERR	; Not done, write more. Don't check for index
				; to prevent being late writing next char.
WREND
	;continue writing to end of track
	BIT	DDPIA	; WAIT FOR
	BMI	*-3	; INDEX PULSE

INDEXERR
	LDA	#$03
	ORA	DDPIA+2	; turn off write and erase
	STA	DDPIA+2
    RTS
	
WR65D ;write track in OS65D format
	; on entry X=#bytes to write for header
	
WR65DH ;1st write track header
	BIT	DDPIA	; WAIT 
	BMI	WR65DHOK	; Didn't find INDEX PULSE (good!)
	JMP	INDEXERR	;got index while trying to write data (error)
WR65DHOK
	LDA	#2
	AND	DDACIA		;test tx ready?	
	BEQ	WR65DH	;not ready?
	LDA	(TDATA),Y
	STA	DDACIA+1	; Write data to floppy
	INY
	DEX
	BNE	WR65DH
	;delay to start of 1st sector of 6.6ms (about 149 bytes) 
	LDX #$05
	JSR DELAY  ;~6.6ms	
	LDA #$00
	STA CURSEC
WRSECT
    LDX CURSEC
	CPX NUMSECTS
	BCS WREND       ;last sector?
	LDA SectPozL,X  ;get sector start data address & lengths
	STA TDATA
	LDA SectPozH,X
	STA TDATA+1
	LDA SectLenL,X
	STA CNT
	INC CNT
	LDA SectLenH,X
	STA CNT+1
    INC CNT+1
	LDA SectLenH-1,X ;previous sector length in pages
	;gap to 1st sector is 4*<delay> where <delay> is 1600us
    ;based on T.R. Berger article in Compute's Gazette #20 Jan 1982 p136 "Actual OS65D disk timing"
	JSR DELAY16 ; ~1600us * A
	LDY #$00
	BCS INDEXERR   ;carry set if index encountered during delay
WR65SECT
	BIT	DDPIA	; WAIT 
	BPL INDEXERR	;got index while trying to write data (error)

WR65NOIDX
	LDA	#2
	AND	DDACIA		;test tx ready?	
	BEQ	WR65SECT	;not ready?
	LDA	(TDATA),Y
	STA	DDACIA+1	; Write data to floppy
	INY
	BNE	WR65N1
	INC	TDATA+1
WR65N1
	DEC CNT
	BNE	WR65SECT
	DEC CNT+1
	BNE WR65SECT
    ;done writing data
	INC CURSEC
	JMP WRSECT

; PRINT IN-LINE STRING
PRINT	
	PLA
	STA	PTRSTOR
	PLA
	STA	PTRSTOR+1
	STY PRYSTOR
	LDY	#1
PRINT1	
	LDA	(PTRSTOR),Y
	BEQ	PRINT2
	JSR	OUTPUT
	INY
	BNE	PRINT1
	INC PTRSTOR+1
	BNE PRINT1
PRINT2	
	TYA
	SEC
	ADC	PTRSTOR
	STA	PTRSTOR
	BCC	PRINT3
	INC	PTRSTOR+1
PRINT3
	CLC
PRYSTOR=*+1
	LDY #$00
	JMP	(PTRSTOR)

; PRINT HEX WORD (A,X)
PHEX	
	JSR	PHEXA
	TXA

; PRINT HEX BYTE (A)
PHEXA	
	PHA
	LSR	A
	LSR	A
	LSR	A
	LSR	A
	JSR	PHEXA1
	PLA
PHEXA1	
	AND	#$F
	ORA	#'0'
	CMP	#'9'+1
	BMI	PHEXA2
	CLC
	ADC	#7
PHEXA2	
	JMP	OUTPUT

; PRINT DECIMAL (A)
PDECA	
	TAX
	LDA	#0

; PRINT DECIMAL (A,X)
PDEC	
	STX	PDEC8
	STA	PDEC8+1
	LDA	#0
	LDX	#2
PDEC1	
	STA	PDEC10,X
	DEX
	BPL	PDEC1
	LDY	#16
	PHP
	SEI
	SED
PDEC2	
	ASL	PDEC8
	ROL	PDEC8+1
	LDX	#2
PDEC3	
	LDA	PDEC10,X
	ADC	PDEC10,X
	STA	PDEC10,X
	DEX
	BPL	PDEC3
	DEY
	BNE	PDEC2
	PLP
	LDX	#6
	LDA	#0
	STA	PDEC9
PDEC4	
	LDY	#3
	LDA	#0
PDEC5	
	ASL	PDEC10+2
	ROL	PDEC10+1
	ROL	PDEC10
	ROL	A
	DEY
	BPL	PDEC5
	TAY
	BNE	PDEC6
	CPX	#1
	BEQ	PDEC6
	LDY	PDEC9
	BEQ	PDEC7
PDEC6	
    INC PDEC9
	ORA	#$30
	JSR	OUTPUT
PDEC7	
    DEX	
	BNE	PDEC4
	RTS
	
;print disk format mesg
PRNFMT
	BIT DSKTYP	;0x80
	BPL PRNFM1
	LDX #$00
	BEQ PRNFMTX
PRNFM1
	BVC PRNFM2	;0x40
	LDX #TYPE65D-TYPEMSG
	BNE PRNFMTX
PRNFM2
	LDX #TYPEUNK-TYPEMSG
	LDA DSKTYP
	BEQ PRNFMTX
PRNFM3
	LDX #TYPEHEX-TYPEMSG
	CMP #$01
	BEQ PRNFMTX
PRNFM4
	LDX #TYPE0-TYPEMSG
	CMP #$02
	BEQ PRNFMTX
PRNFM5
	RTS
PRNFMTX
	LDA TYPEMSG,X
    BEQ PRNFM5
    JSR OUTPUT
    INX
    BNE PRNFMTX	
	
TYPEMSG=*	;disk type messages 
TYPE65U
	.BYTE ' 65U ',0
TYPE65D
	.BYTE ' 65D ',0
TYPE0
	.BYTE ' boot ',0
TYPEHEX
	.BYTE ' HexDos ',0
TYPEUNK
	.BYTE ' ?? ',0

SCRNCLR
	LDX #$32
	JSR CRLF
	DEX
	BNE SCRNCLR+2
	RTS
CRLF	
	LDA	#CR
	JSR	OUTPUT
	LDA	#LF
	JMP	OUTPUT

ANYKEY
	JSR	PRINT
	.BYTE	CR,LF
	.BYTE	'Press any key when ready',CR,LF,0
	JMP	INKEY

.IFN 0 <
CHKERR
	BIT USEXMOD
	BMI READMS1 
	JSR	PRINT
	.BYTE	' BAD CHECKSUM ',0
	RTS
>
; PRINT "READING..." MESSAGE WHEN DOING ASCII HEX DUMPS
READMSG	
	LDA USEXMOD
	BMI READMS1 ;no mesg with xmodem
	JSR	PRINT
	.BYTE	CR,LF,CR,LF
	.BYTE	';READING TRACK ',0
	LDA TRK
	JMP	PDECA
READMS1
	RTS

SCRNMSG
    .BYTE 'OSI DISK TOOL ON SERIAL NOW',0

DRWMENU
	 JSR PRINT
	.BYTE $D,$A,$D,$A
	.BYTE ' OSI Serial Disk Tool ',$D,$A          
	.BYTE '----------------------',$D,$A          
	.BYTE '1. Dump Raw Image',$D,$A
;	.BYTE '2. Dump w/ Error Check',$D,$A
	.BYTE '3. Receive Disk Xmodem',$D,$A
	.BYTE '4. Select Drive',$D,$A      
	.BYTE '5. Set Drive Type',$D,$A      
	.BYTE '6. Set Serial Divisor',$D,$A
	.BYTE '7. Xmodem/Hex xfer opts',$D,$A 
	.BYTE '8. Exit',$D,$A 
	.BYTE 'L. View log',$D,$A
	.BYTE $D,$A
	.BYTE 'Drv=',$0
	LDA DRVNUM
	CLC
	ADC #$41
	JSR	OUTPUT
	LDA	MAXPAG
	CMP #$0A
	BCS DRWMN1
	JSR PRINT
	.BYTE '/5.25',0
	BCC DRWMN2
DRWMN1
	JSR PRINT
	.BYTE '/8 ',0
DRWMN2
	JSR PRINT
	.BYTE ' Baud@1',0
   	LDA ACIACTL
	LSR A
	BCC DRWMN3
	JSR PRINT
	.BYTE 'x ',0
	JMP DRWMN4
DRWMN3
	JSR PRINT
	.BYTE '6x ',0
DRWMN4
	BIT USEXMOD
	BPL DRWMN5
	BVC DRWMN8
	JSR PRINT
	.BYTE 'Xmdm ',0
	BCC DRWMN6
DRWMN8
	JSR PRINT
	.BYTE 'Xmdm/crc ',0
	BCC DRWMN6
DRWMN5	
	JSR PRINT
	.BYTE 'Hex ',0
DRWMN6
    LDA WRITERR
	BEQ DRWMN7
	JSR PRINT
	.BYTE $A,$D, '**Errors Logged**', 0
DRWMN7
	JSR PRINT
	 .BYTE $D,$A,'--->',0
	JMP FLUSHM

INZACIA
	BIT MACHINE
	BMI INZAC1
    LDA #$03	;reset ACIA
	STA C2ACIA
	LDA ACIACTL	;
	STA C2ACIA
	RTS
INZAC1			;reset C1 ACIA
	LDA #$03
	STA C1ACIA
	LDA ACIACTL
	STA C1ACIA
	RTS
	
Get_Chrs
	JSR Get_Chr
	BCC Get_Chrs-1
	PHP
	PHA
	LDA #'R
	JSR LOGTEXT
	PLA
	JSR LOGTEXT
	PLP
	RTS

Get_Chr
FRACIANW  ; read from ACIA no wait	carry clear when no data
	BIT MACHINE
	BMI FRAC1NW
	LDA C2ACIA
	LSR A
	BCC FRAC1NW-1
	LDA C2ACIA+1
	RTS
FRAC1NW
	LDA C1ACIA
	LSR A    
	BCC FRAC1NW-1 
	LDA C1ACIA+1
	RTS

INKEY
FRACIA   	 ;read from ACIA carry set on abort return value in A
	BIT MACHINE
	BMI FRAC1
FRSER			; read from C3
	LDA C2ACIA
	LSR A    
	BCC FRACIA 
	LDA C2ACIA+1
	CLC      
ACIARET
	RTS
FRAC1			;read from C1
	LDA C1ACIA
	LSR A    
	BCC FRACIA 
	LDA C1ACIA+1
	CLC       
	RTS
	
VIDOFF			;disable print output to video screen
	LDA #$80
	.BYTE $2C 
SEROFF
    LDA #$40	;disable print output to serial
	ORA OUTFLAG
	STA OUTFLAG
	RTS
	
VIDON
	LDA #$7F	;enable print output to video screen
	.BYTE $2C
SERON
	LDA #$BF
	AND OUTFLAG	;enable serial print output
	STA OUTFLAG
	RTS

Put_Chrs
	PHA
	LDA #'S
	JSR LOGTEXT
	PLA
	JSR LOGTEXT
	JMP Put_Chr

OUTLOG
	JMP LOGTEXT  ;when video only is selected, log text for serial system later
OUTPUT
	BIT OUTFLAG ;0x80 = no video
	BMI *+5
	JSR VIDOUT
	BIT OUTFLAG
	BVS OUTLOG  ;0x40 = no serial
Put_Chr
TOACIA    ;output to ACIA C1 or C2 machine
	PHA
	BIT MACHINE
	BMI TOAC1
TOACIA1	       
	LDA C2ACIA   ;wait for TxEmpty
	LSR A     
	LSR A     
	BCC TOACIA1   
	PLA       
	STA C2ACIA+1 
	RTS       
TOAC1
	LDA C1ACIA 
	LSR A     
	LSR A     
	BCC TOAC1   
	PLA       
	STA C1ACIA+1 
	RTS 

	
	; Video display output routine
	; video init for SYN600, SYNMON, CEGMON
	; $FFE0 - cursor start
	; $FFE1 - line len-1   (wrap position)
	; $FFE2 - 00 = 1K vid, 01=2K vid otherwise serial or other ROM
	; FFE0 65, 17, 00 (c1 ROM, cegmon)
	; FFE0 40, 3F, 01 (c2 ROM)
	; FFE0 4D, 2F, 01 (C1E cegmon 64x32)
VIDINIT
	LDA #$00
	STA VIDOFFSET
	STA VIDDST
	STA OUTFLAG
	LDX $FFE2
	BEQ C1VID ;1k screen
	DEX
	BEQ C2VID
	
WHATVID	;assume C2-like video
	LDA #$00
	STA VIDCR+1 ;cursor start
	STA VIDOFFSET
	LDA #$40	;linelen
	STA VIDXMAX
	LDX #$D7    ;bottom line hi
	LDY #$80    ;bottom line low
	BNE VIDINI2
	
C2VID			;C2VID	2k screen
	LDA #$40	;linelen
	LDX #$D7    ;bottom line hi
	BNE VIDINI1
	
	
C1VID	;C1VID  1K screen
	LDA #$20
	LDX #$D3

VIDINI1	;read video settings from ROM
	PHA	
	LDA $FFE0
	TAY
	AND #$0F
	STA VIDCR+1 ;cursor start
	SEC
	ADC $FFE1
	STA VIDXMAX
	TYA
	AND #$F0
	TAY         ;bottom line low
	STA VIDOFFSET
	PLA

VIDINI2	
	STA VIDXLEN
	STA VIDSRC

	STX VIDCLR+2  ;D3E0 or D780
	STY VIDCLR+1  
	STX VIDFIX1+1 ;D3E0 or D780
	STY VIDFIX1
	LDA #$D0
	STA VIDSRC+1  ;init to D040 or D020
	STA VIDDST+1  ;init to D000
	RTS
	
VIDOUT            ;entry point for screen printer, not ROMable 
    STA AHOLD
	STY YHOLD 
	CMP #CR
	BEQ VIDCR
	CMP #LF
	BEQ VIDLF
	LDY VIDOFFSET
VIDFIX1=*+1
	STA $D6C0,Y
	INY
	STY VIDOFFSET
	CPY VIDXMAX
	BNE VIDRETY
	JSR VIDCR
	JMP VIDLF
	
VIDRETY
YHOLD = *+1
	LDY #$00
AHOLD = *+1
	LDA #$0D
	RTS
VIDCR
	LDA #0
	STA VIDOFFSET
	LDA AHOLD
	RTS

VIDLF
	LDY #$00
VIDLF1
	LDA (VIDSRC),Y  ;copy row by row
	STA (VIDDST),Y
	INY
	CPY VIDXLEN
	BNE VIDLF1
	TYA
	CLC
	ADC VIDSRC
	STA VIDSRC
	BCC VIDLF2
	INC VIDSRC+1
VIDLF2
	TYA
	CLC
	ADC VIDDST
	STA VIDDST
	BCC VIDLF3
	INC VIDDST+1
VIDLF3	

	LDA VIDDST+1
	CMP VIDCLR+2
	BNE VIDLF1-2
	LDA VIDDST
	CMP VIDCLR+1
	BNE VIDLF1-2

	LDA #$D0
	STA VIDSRC+1
	STA VIDDST+1
	LDA #$00
	STA VIDDST
	LDA VIDXLEN
	STA VIDSRC

	LDA #' '
VIDCLR
	STA $D6C0,Y	; Clear last line
	INY
	CPY VIDXLEN
	BNE VIDCLR
	INY
	JMP VIDRETY
	
	;clear log	
LOGRESET
	LDA #>LOGBUFF
	STA LOGMEM+1
	LDA #<LOGBUFF
	STA LOGMEM
	LDY #$00
	TYA
LOGR2
	STA (LOGMEM),Y
	INY
	BNE LOGR2
	INC LOGMEM+1
	LDA LOGMEM+1
	CMP HIGHMEM
	LDA #$00
	BCC LOGR2
	LDA #>LOGBUFF
	STA LOGMEM+1
	
	LDA #$00
	STA WRITERR
	RTS
	
	;add character to log until full
LOGTEXT
	STY LOGY
	LDY LOGMEM+1
	DEY				;allow up to xxFF
	CPY HIGHMEM
	BCS LOGFUL
	LDY #$00
	STA (LOGMEM),Y
	INC LOGMEM
	BNE LOGFUL
	INC LOGMEM+1
LOGFUL
LOGY=*+1
	LDY #$00
	RTS
	
	;Send log to serial port
LOGPRINT
	LDA #>LOGBUFF
	STA TMP+1
	LDA #<LOGBUFF
	STA TMP
	JMP LOGPR2
LOGPR1
	LDY #$00
	LDA (TMP),Y
	JSR TOACIA
	INC TMP
	BNE LOGPR2
	INC TMP+1
LOGPR2
    LDA TMP
	CMP LOGMEM
	BNE LOGPR1
	LDA TMP+1
	CMP LOGMEM+1
	BNE LOGPR1
    RTS
	
	;locate top of memory below disk controller address
TOPMEM	
	LDY #>LOGBUFF
TOPM1
	JSR PAGEMEM
	BNE TOPM2
	INY
	CPY #$C0
	BNE TOPM1	
TOPM2
	DEY
	STY HIGHMEM
	RTS
	
PAGEMEM
	STY TMP+1	; Non Destructive Mem Test, on entry Y= Target Page uses Loc TMP, TMP+1, CNT
	LDY #$FF	; Tests byte at $yyFF where yy=target page, return Zero flag if OK
	STY TMP
	INY
	LDA (TMP),Y
	TAX
	AND #$3F
	EOR #$3F
	STA (TMP),Y
	STA CNT
	LDA (TMP),Y
	AND #$3F
	CMP CNT
	PHP
	TXA
	STA (TMP),Y
	LDY TMP+1
	PLP
	RTS

;----------------------------------------------------------------------------
; XMODEM/CRC Sender for the 65C02
;
; packet is 128 bytes, last packet is zero padded 
; packet consists of <SOH=$01><blk# (starting at 1)><255-block#>< 128 data bytes><crclo><crc hi> for 133 bytes
; "C" is used to acknowledge & start transfer
;
; By Daryl Rictor Aug 2002
;
; A simple file transfer program to allow upload from the SBC to a 
; console device utilizing the x-modem/CRC transfer protocol.  Requires just
; under 1k of either RAM or ROM, 132 bytes of RAM for the receive buffer,
; and 8 bytes of zero page RAM for variable storage.
;
;**************************************************************************
; This implementation of XMODEM/CRC does NOT conform strictly to the 
; XMODEM protocol standard in that it (1) does not accurately time character
; reception or (2) fall back to the Checksum mode.
; (1) For timing, it uses a crude timing loop to provide approximate
; delays.  These have been calibrated against a 1MHz CPU clock.  I have
; found that CPU clock speed of up to 5MHz also work but may not in
; every case.  Windows HyperTerminal worked quite well at both speeds!
;
; (2) Most modern terminal programs support XMODEM/CRC which can detect a
; wider range of transmission errors so the fallback to the simple checksum
; calculation was not implemented to save space.
;**************************************************************************
; xNOT IN THIS PROGRAM
; xFiles uploaded via XMODEM-CRC will have the load address contained in
; xthe first two bytes in little-endian format:  
; x FIRST BLOCK
; x    offset(0) = lo(load start address),
; x    offset(1) = hi(load start address)
; x    offset(2) = data byte (0)
; x    offset(n) = data byte (n-2)
;
; Subsequent blocks
;     offset(n) = data byte (n)
;
; One note, XMODEM send 128 byte blocks.  If the block of memory that
; you wish to save is smaller than the 128 byte block boundary, then
; the last block will be padded with zeros.  Upon reloading, the
; data will be written back to the original location.  In addition, the
; padded zeros WILL also be written into RAM, which could overwrite other
; data.   
;
;-------------------------- The Code ----------------------------
;
; zero page variables (adjust these to suit your needs)
;
;
;xmdata      =   $35
;lastblk		=	xmdata			; flag for last block
;blkno		=	xmdata+1		; block number 
;Xerrcnt		=	xmdata+2		; error counter 10 is the limit

;crc			=	xmdata+3		; CRC lo byte  (two byte variable)
;crch		=	xmdata+4		; CRC hi byte  

;ptr			=	xmdata+5		; data pointer (two byte variable)
;ptrh		=	xmdata+6		;   "    "

;eofp		=	xmdata+7		; end of file address pointer (2 bytes)
;eofph		=	xmdata+8		;  "	"	"	"

;Xretry		=	xmdata+9		; retry counter 
;Xretry2		=	xmdata+10		; 2nd counter
;lastdata     =   xmdata+11       ;flag buffer of file

;
;
; non-zero page variables and buffers
;
;
;Xbuff		=	$4000      	; temp 133 byte receive buffer 
;					;(place anywhere, page aligned)
;
;
;  tables and constants
;
;
; The crclo & crchi labels are used to point to a lookup table to calculate
; the CRC for the 128 byte data blocks.  There are two implementations of these
; tables.  One is to use the tables included (defined towards the end of this
; file) and the other is to build them at run-time.  If building at run-time,
; then these two labels will need to be un-commented and declared in RAM.
;
;crclo		=	$7D00      	; Two 256-byte tables for quick lookup
;crchi		= 	$7E00      	; (should be page-aligned for speed)
;
;
;
; XMODEM Control Character Constants


;
;^^^^^^^^^^^^^^^^^^^^^^ Start of Program ^^^^^^^^^^^^^^^^^^^^^^
;
; Xmodem/CRC download routine
; By Daryl Rictor, August 8, 2002
;
; v0.1  code creation
;
;
;		*= 	$7800		; Start of program (adjust to your needs)
;
; Enter this routine with the beginning address stored in the zero page address
; pointed to by TDATA & TDATA+1 and the ending address stored in the zero page address
; pointed to by eofp & eofph.
;
;
XResume
		bit	lastdata
		bvc	XModemStart	 ;bit 6 = 0, means its 1st block
		ldx	Xsavem
		ldy	#$00
		STY Xerrcnt    
		;JSR GetByte
		JSR FlushQ		;MAC/Linux lrzsz sends us NAKs when we we're busy
		CMP #CAN
		BEQ XAbort
		CMP #ESC
		BEQ XAbort
		
		BNE	LdBuff1
XAbort
		Jmp GotAbort

;XMODEM Send 		
XModemStart
		lda	#$40		;bit tst V = 1, means not 1st block next time
		sta	lastdata
		jsr	PrintMsg	; send prompt and info
		lda	#$00		;
		sta	Xerrcnt		; error counter set to 0
		sta	lastblk		; set flag to false
		lda	#$01		;
		sta	blkno		; set block # to 1
Wait4CRC
		lda	#$ff		; 3 seconds
		sta	Xretry2		;
		jsr	GetByte		;
		bcc	Wait4CRC	; wait for something to come in...
		cmp	#'C			; is it the "C" to start a CRC xfer?
		beq	SetCRC		; yes
		cmp #NAK
		BNE Wait1
		LDA #$C0
		.BYTE $2C
SetCRC
		LDA #$80
		STA USEXMOD		;receiver wants std xmodem
		JMP SetstAddr
Wait1
		cmp #CAN
		beq *+6
		cmp	#ESC		; is it a cancel? <Esc> Key
		bne	Wait4CRC	; No, wait for another character
		jmp	PrtAbort	; Print abort msg and exit
SetstAddr
		ldy	#$00		; init data block offset to 0
		
		lda	#$01		; manually load Blk #1	
		sta	Xbuff		; into 1st byte
		lda	#$FE		; load 1's comp of block #	
		sta	Xbuff+1		; into 2nd byte
	;when transferring disk images, we don't send load address
		;ldx	#$04		; preload X to Receive buffer
		;lda	TDATA			; load low byte of start address		
		;sta	Xbuff+2		; into 3rd byte	
		;lda	TDATA+1		; load hi byte of start address		
		;sta	Xbuff+3		; into 4th byte
		ldx	#$02        ;our buffer only has 2 byte preloaded now
		jmp	Ldbuff1		; jump into buffer load routine

LdBuffer
		lda	lastblk		; Was the last block sent?
		beq	LdBuff0		; no, send the next one	
		jmp Done		; yes, we're done
LdBuff0
		ldx	#$02		; init pointers
		ldy	#$00		;
		inc	Blkno		; inc block counter
		lda	Blkno		; 
		sta	Xbuff		; save in 1st byte of buffer
		eor	#$FF		; 
		sta	Xbuff+1		; save 1's comp of blkno next

LdBuff1
		lda	(TDATA),y		; save 128 bytes of data
		sta	Xbuff,x		;
LdBuff2
		lda	TDATA+1 	;
		cmp	EOFPH		;
		bcc	LdBuff4		;
		lda	TDATA		;
		cmp	EOFP		; Are we at the last address?
		bcc	LdBuff4		; no, inc pointer and continue

		bit	lastdata	; set when last transfer block
		bmi	LdBuf2b
		stx	Xsavem      ;save output buffer pos for next time we are called
		clc
		rts
LdBuf2b                 ;no more blocks to send ; if we're at the start of a block finish
		CPX #$02        ;otherwise continue to pad to end
		BNE LdBuf2a
LdBuf2c
		LDA #EOT
		jsr	Put_Chr
		jsr	GetByte		
		CMP #ACK
		BEQ LdBuf2d
		CMP #NAK
		BEQ LdBuf2c
		CMP #CAN
		BEQ LdBuf2d
		CMP #ESC
		BNE LdBuf2c
LdBuf2d
		JMP Done
LdBuf2a
		inc	lastblk		; Yes, Set last byte flag
LdBuff3
		inx			;
		cpx	#$82		; Are we at the end of the 128 byte block?
		beq	DoCRC		; Yes, calc CRC
		lda	#$00		; Fill rest of 128 bytes with $00
		sta	Xbuff,x		;
		beq	LdBuff3		; Branch always

LdBuff4
		inc	TDATA		; Inc address pointer
		bne	LdBuff5		;
		inc	TDATA+1		;
LdBuff5
		inx			;
		cpx	#$82		; last byte in block?
		bne	LdBuff1		; no, get the next

DoCRC
		LDY #$82
		JSR CalcCRC
		lda	crch		; save Hi byte of CRC to buffer
		sta	Xbuff,y		;
		iny			;
		lda	crc			; save lo byte of CRC to buffer
		sta	Xbuff,y		;
Resend
		ldx	#$00		;
		lda	#SOH		; Send start block command
		jsr	Put_Chr		;
SendBlk
		lda	Xbuff,x		; Send 133 bytes in buffer to the console
		jsr	Put_Chr	;
		JSR LOGTEXT
		inx			;
		CPX #$84
		BCC SendBlk		
		BIT USEXMOD		;using CRC mode?
		BVS sendbl2		;no checksum, so branch
sendbl1
		cpx	#$84		; last byte?
		bne	SendBlk		; no, get next
sendbl2
		lda	#$FF		; yes, set 3 second delay 
		sta	Xretry2		; and
		jsr	GetByte		; Wait for Ack/Nack
		bcc	Seterror	; No chr received after 3 seconds, resend
		cmp	#NAK		; 
		beq	Seterror	; NAK, inc errors and resend
		cmp	#ESC		;
		beq	PrtAbort	; Esc pressed to abort
		cmp	#ACK		; Chr received... is it:
		bne SetError
		LDA #$00		;reset error count
		STA Xerrcnt
		JMP LdBuffer	; ACK, send next block				
Seterror
		inc	Xerrcnt		; Inc error counter
		lda	Xerrcnt		; 
		cmp	#$0A		; are there 10 errors in a row? (Xmodem spec for failure)
		bcc	Resend		; no, resend block
		JSR PRINT
		.byte 	CAN,CAN,CAN,'Errors Exceeded!',CR,LF,0
		sec
		rts		
PrtAbort
		jsr	Flush		; yes, too many errors, flush buffer,
		JSR PRINT
		.byte 	CAN,CAN,CAN,'Abort Received!',CR,LF,0
		sec
		rts		
Done
		jmp	Print_Good	; All Done..Print msg and exit

		
Print_Err	
		ldx	#$00		; PRINT Error message
PrtErr1
		lda ErrMsg,x
		beq	PrtErr2
		jsr	Put_Chr
		inx
		bne	PrtErr1
PrtErr2
		rts
ErrMsg
		.byte CAN,CAN,CAN, "Transfer Error!",$A,$D,0
ErrWrongBlk
        .byte CAN,CAN,CAN, "Wrong Blk# Error",$A,$D,0
ErrBlkNum
		.byte CAN,CAN,CAN, "Blk Mismatch",$A,$D,0
	
; X-Modem Receive Protocol by Daryl Rictor
;	
;entry point to receive disk via xmodem		
XRcvDisk
		jsr	PrintMsg	; send prompt and info
		lda	#$01
		sta	blkno		; set block # to 1
		ldy	#$00
		STY Xerrcnt     ;errors per file		
		;sta	bflag		; set flag to get address from block 1
		JSR FlushQ
StartCrc
		BIT USEXMOD
		BVC startcr1 
		lda #NAK		;start Checksum mode?
		.BYTE $2C		;bit test mask of LDA
startcr1
		lda	#'C 		; "C" start with CRC mode
		ldy #3
		jsr	Put_Chr	; send it
		lda	#$FF	
		sta	Xretry2		; set loop counter for ~3 sec delay
		lda	#$00
        sta	crc
		sta	crch		; init CRC value	
startcr2
		jsr	GetByte		; wait for input       	
		bcs startcr3
		DEY
		bpl startcr2
		bmi startCrc	; resend "C"
startcr3
		CMP #ESC
		beq GotAbort
		CMP #CAN
		beq GotAbort
		CMP #SOH
		BNE StartCr2
		BEQ BegBlk

GotAbort
		JSR PrtAbort
		lda	#ABORTRECV		; Error code in "A" of desired
		sec
		rts		
StartBlk
		lda	#$FF		; 
		sta	Xretry2		; set loop counter for ~3 sec delay
		jsr	GetByte		; get first byte of block
		bcc	StartBlk	; timed out, keep waiting...


GotByte1
		cmp	#SOH		; start of block?
		beq	BegBlk		; yes
		cmp	#EOT		;
		bne	BadCrc		; Not SOH or EOT, so flush buffer & send NAK	
		jmp	RDone		; EOT - all done!
BegBlk
		ldx	#$00
GetBlk
		lda	#$ff		; 3 sec window to receive characters
		sta Xretry2		;
GetBlk1
		jsr	GetByte		; get next character
		bcc	BadCrc		; chr rcv error, flush and send NAK
GetBlk2
		sta	Xbuff,x		; good char, save it in the rcv buffer
		inx				; inc buffer pointer
        BIT USEXMOD	
		BVC GetBlk2a
		cpx #$83		;checksum mode
		bne GetBlk1
		beq GetBlk2b
GetBlk2a				;crc mode
		cpx	#$84		; <01> <FE> <128 bytes> <CRCH> <CRCL>
		bne	GetBlk		; get 132 characters
GetBlk2b
		ldx	#$00		;
		lda	Xbuff,x		; get block # from buffer
		cmp	blkno		; compare to expected block #	
		beq	GoodBlk1	; matched!
		bcc XIgnorBlk   ;whatever we already had this one
		
		ldx #ErrWrongBlk - ErrMsg
		jsr	PrtErr1	    ; Unexpected block number - abort
		jsr	Flush		; mismatched - flush buffer and then do BRK
		lda	#WRONGBLK	; put error code in "A" if desired
		sec				; unexpected block # - fatal error - BRK or RTS
		rts
GoodBlk1
		eor	#$ff		; 1's comp of block #
		inx				;
		cmp	Xbuff,x		; compare with expected 1's comp of block #
		beq	GoodBlk2 	; matched!
		bne BadCrc
		;ldx #ErrBlkNum - ErrMsg
		;jsr	PrtErr1		; Unexpected block number - abort	
		;jsr Flush		; mismatched - flush buffer and then do BRK
		;lda	#MISMATCH	; put error code in "A" if desired
						; bad 1's comp of block#
		;sec
		;rts
GoodBlk2
		jsr	CalcCRC		; calc CRC
		lda	Xbuff,y		; get hi CRC from buffer
		cmp	crch		; compare to calculated hi CRC
		bne	BadCrc		; bad crc, send NAK
		BIT USEXMOD		; using checksum mode?
		BVS GoodCrc		;yup, branch
		iny				;
		lda	Xbuff,y		; get lo CRC from buffer
		cmp	crc			; compare to calculated lo CRC
		beq	GoodCrc		; good CRC
BadCrc
		inc Xerrcnt
		jsr	Flushm		; flush the input port
		lda	#NAK		;
		jsr	Put_Chr	; send NAK to resend block
		jmp	StartBlk	; start over, get the block again			
GoodCrc
		ldx	#$02		;
		;lda	blkno		; get the block number
		;cmp	#$01		; 1st block?
		;bne	CopyBlk		; no, copy all 128 bytes
		;lda	bflag		; is it really block 1, not block 257, 513 etc.
		;beq	CopyBlk		; no, copy all 128 bytes
		;lda	Xbuff,x		; get target address from 1st 2 bytes of blk 1
		;sta	TDATA		; save lo address
		;inx				;
		;lda	Xbuff,x		; get hi address
		;sta	TDATA+1		; save it
		;inx				; point to first byte of data
		;dec	bflag		; set the flag so we won't get another address		
CopyBlk
		ldy	#$00		; set offset to zero
CopyBlk3
		lda	Xbuff,x		; get data byte from buffer
		sta	(TDATA),y	; save to target
		inc	TDATA		; point to next address
		bne	CopyBlk4	; did it step over page boundary?
		inc	TDATA+1		; adjust high address for page crossing
CopyBlk4
		inx				; point to next data byte
		cpx	#$82		; is it the last byte
		bne	CopyBlk3	; no, get the next one
IncBlk

		lda TDATA+1
		cmp EOFPH
		bne XNextBlk
		lda TDATA
		cmp EOFP
		bne XNextBlk		
		jmp XWriteTrk	;exit xmodem to write trk now
		
		
XNextBlk		        ;receive next block/track
		inc	blkno		; done.  Inc the block #
XIgnorBlk
		lda	#ACK		; send ACK
		jsr	Put_Chr	;
		jmp	StartBlk	; get next block

RDone
		lda	#ACK		; last block, send ACK and exit.
		jsr	Put_Chr	;
		jsr	Flush		; get leftover characters, if any
		JMP Done
;		
;
;^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
;
; subroutines
;
;					;
GetByte	
        lda #$00		; wait for chr input and cycle timing loop
		sta	Xretry		; set low value of timing loop
StartCrcLp	
		jsr	Get_chr 	; get chr from serial port, don't wait 
		bcs	GetByte1	; got one, so exit
		EOR	$FF,X       ; waste some time
		EOR	$FF,X	
		dec	Xretry		; no character received, so dec counter
		bne	StartCrcLp	;
		dec	Xretry2		; dec hi byte of counter
		bne	StartCrcLp	; look for character again
		clc			    ; if loop times out, CLC, else SEC and return
GetByte1
		rts			    ; with character in "A"
;
Flushm    ;flush lite
	    lda #$20
		.BYTE $2c   ;bit
Flush		
		lda	#$70		; flush receive buffer
		sta	Xretry2		; flush until empty for ~1 sec.
Flush1		
		jsr	GetByte		; read the port
		bcs	Flush		; if chr recvd, wait for another
		rts			; else done
;
FlushQ
	jsr Get_chr
	bcc FlushQ1
	CMP #NAK
	BEQ FlushQ
	RTS
FlushQ1
	LDA #$00
	rts

PrintMsg
		JSR PRINT
		.BYTE	'Begin XMODEM/',0
		BIT USEXMOD
		BVS Printm1
		JSR PRINT
		.BYTE 'CRC',0
		BCC Printm2
Printm1
		JSR PRINT
		.BYTE 'Checksum',0
Printm2
		JSR PRINT
		.BYTE   ' transfer.  Press <ESC> to abort...'
		.BYTE  	CR, LF, 0		
		RTS

Print_Good
		JSR PRINT
		.BYTE	EOT,EOT,EOT,CR,LF,CR,LF   ; send end of file three times
		.BYTE 	'Transfer Complete!'
		.BYTE  	CR, LF, 0
		CLC	
		RTS

CalcChk
	LDY #$02
	LDA #$00
CalcCh1
    CLC
	ADC	Xbuff,y		;
	iny			;
	cpy	#$82		; done yet?
	bne	CalcCh1
	STA crch		;value of checksum byte use when creating send block
	RTS		
;=========================================================================
;
;
;  CRC subroutines 
;
;
CalcCRC
		BIT USEXMOD
		BVS CalcChk
		lda	#$00		; yes, calculate the CRC for the 128 bytes
		sta	crc		;
		sta	crch		;
		ldy	#$02		;
CalcCRC1
		lda	Xbuff,y		;
		jsr	UpdCRC		;		
		iny			;
		cpy	#$82		; done yet?
		bne	CalcCRC1	; no, get next
		RTS

UpdCrc		
		eor 	crc+1 		; Quick CRC computation with lookup tables
		tax		 	; updates the two bytes at crc & crc+1
		lda 	crc		; with the byte send in the "A" register
		eor 	CRCHI,X
		sta 	crc+1
		lda 	CRCLO,X
		sta 	crc
		rts
;
; Alternate solution is to build the two lookup tables at run-time.  This might
; be desirable if the program is running from ram to reduce binary upload time.
; The following code generates the data for the lookup tables.  You would need to
; un-comment the variable declarations for crclo & crchi in the Tables and Constants
; section above and call this routine to build the tables before calling the
; "xmodem" routine.
;
;MAKECRCTABLE
;		ldx 	#$00
;		LDA	#$00
;zeroloop	sta 	crclo,x
;		sta 	crchi,x
;		inx
;		bne	zeroloop
;		ldx	#$00
;fetch		txa
;		eor	crchi,x
;		sta	crchi,x
;		ldy	#$08
;fetch1		asl	crclo,x
;		rol	crchi,x
;		bcc	fetch2
;		lda	crchi,x
;		eor	#$10
;		sta	crchi,x
;		lda	crclo,x
;		eor	#$21
;		sta	crclo,x
;fetch2		dey
;		bne	fetch1
;		inx
;		bne	fetch
;		rts
;
; The following tables are used to calculate the CRC for the 128 bytes
; in the xmodem data blocks.  You can use these tables if you plan to 
; store this program in ROM.  If you choose to build them at run-time, 
; then just delete them and define the two labels: crclo & crchi.
;
; low byte CRC lookup table (should be page aligned)

*= *+255/256*256 ;page align the following tables
crclo
 .byte $00,$21,$42,$63,$84,$A5,$C6,$E7,$08,$29,$4A,$6B,$8C,$AD,$CE,$EF
 .byte $31,$10,$73,$52,$B5,$94,$F7,$D6,$39,$18,$7B,$5A,$BD,$9C,$FF,$DE
 .byte $62,$43,$20,$01,$E6,$C7,$A4,$85,$6A,$4B,$28,$09,$EE,$CF,$AC,$8D
 .byte $53,$72,$11,$30,$D7,$F6,$95,$B4,$5B,$7A,$19,$38,$DF,$FE,$9D,$BC
 .byte $C4,$E5,$86,$A7,$40,$61,$02,$23,$CC,$ED,$8E,$AF,$48,$69,$0A,$2B
 .byte $F5,$D4,$B7,$96,$71,$50,$33,$12,$FD,$DC,$BF,$9E,$79,$58,$3B,$1A
 .byte $A6,$87,$E4,$C5,$22,$03,$60,$41,$AE,$8F,$EC,$CD,$2A,$0B,$68,$49
 .byte $97,$B6,$D5,$F4,$13,$32,$51,$70,$9F,$BE,$DD,$FC,$1B,$3A,$59,$78
 .byte $88,$A9,$CA,$EB,$0C,$2D,$4E,$6F,$80,$A1,$C2,$E3,$04,$25,$46,$67
 .byte $B9,$98,$FB,$DA,$3D,$1C,$7F,$5E,$B1,$90,$F3,$D2,$35,$14,$77,$56
 .byte $EA,$CB,$A8,$89,$6E,$4F,$2C,$0D,$E2,$C3,$A0,$81,$66,$47,$24,$05
 .byte $DB,$FA,$99,$B8,$5F,$7E,$1D,$3C,$D3,$F2,$91,$B0,$57,$76,$15,$34
 .byte $4C,$6D,$0E,$2F,$C8,$E9,$8A,$AB,$44,$65,$06,$27,$C0,$E1,$82,$A3
 .byte $7D,$5C,$3F,$1E,$F9,$D8,$BB,$9A,$75,$54,$37,$16,$F1,$D0,$B3,$92
 .byte $2E,$0F,$6C,$4D,$AA,$8B,$E8,$C9,$26,$07,$64,$45,$A2,$83,$E0,$C1
 .byte $1F,$3E,$5D,$7C,$9B,$BA,$D9,$F8,$17,$36,$55,$74,$93,$B2,$D1,$F0 

; hi byte CRC lookup table (should be page aligned)
crchi
 .byte $00,$10,$20,$30,$40,$50,$60,$70,$81,$91,$A1,$B1,$C1,$D1,$E1,$F1
 .byte $12,$02,$32,$22,$52,$42,$72,$62,$93,$83,$B3,$A3,$D3,$C3,$F3,$E3
 .byte $24,$34,$04,$14,$64,$74,$44,$54,$A5,$B5,$85,$95,$E5,$F5,$C5,$D5
 .byte $36,$26,$16,$06,$76,$66,$56,$46,$B7,$A7,$97,$87,$F7,$E7,$D7,$C7
 .byte $48,$58,$68,$78,$08,$18,$28,$38,$C9,$D9,$E9,$F9,$89,$99,$A9,$B9
 .byte $5A,$4A,$7A,$6A,$1A,$0A,$3A,$2A,$DB,$CB,$FB,$EB,$9B,$8B,$BB,$AB
 .byte $6C,$7C,$4C,$5C,$2C,$3C,$0C,$1C,$ED,$FD,$CD,$DD,$AD,$BD,$8D,$9D
 .byte $7E,$6E,$5E,$4E,$3E,$2E,$1E,$0E,$FF,$EF,$DF,$CF,$BF,$AF,$9F,$8F
 .byte $91,$81,$B1,$A1,$D1,$C1,$F1,$E1,$10,$00,$30,$20,$50,$40,$70,$60
 .byte $83,$93,$A3,$B3,$C3,$D3,$E3,$F3,$02,$12,$22,$32,$42,$52,$62,$72
 .byte $B5,$A5,$95,$85,$F5,$E5,$D5,$C5,$34,$24,$14,$04,$74,$64,$54,$44
 .byte $A7,$B7,$87,$97,$E7,$F7,$C7,$D7,$26,$36,$06,$16,$66,$76,$46,$56
 .byte $D9,$C9,$F9,$E9,$99,$89,$B9,$A9,$58,$48,$78,$68,$18,$08,$38,$28
 .byte $CB,$DB,$EB,$FB,$8B,$9B,$AB,$BB,$4A,$5A,$6A,$7A,$0A,$1A,$2A,$3A
 .byte $FD,$ED,$DD,$CD,$BD,$AD,$9D,$8D,$7C,$6C,$5C,$4C,$3C,$2C,$1C,$0C
 .byte $EF,$FF,$CF,$DF,$AF,$BF,$8F,$9F,$6E,$7E,$4E,$5E,$2E,$3E,$0E,$1E
;
;
; End of Xmodem
;
BUFFER	=	*
*=*+$0F00 ;space for track buffer
VBUFF = *
*=*+$0F00 ;space for verify buffer
LOGBUFF = *  ;diagnostic log to top of mem

.END

